@@ -208,7 +208,7 @@ let run_solver ~show_preferences ~trust_db tools ?test_callback ?(systray=false)
         let box = Component_box.create tools ~trust_db reqs role ~recalculate ~select_versions_tab ~watcher in
         component_boxes := !component_boxes |> RoleMap.add role box;
         box#dialog#connect#destroy ==> (fun () -> component_boxes := !component_boxes |> RoleMap.remove role);
-        box#update;
+        box#update (Some role);
         box#dialog#show () in
 
   let component_tree = Component_tree.build_tree_view config ~parent:dialog ~packing:widgets.swin#add
@@ -235,7 +235,7 @@ let run_solver ~show_preferences ~trust_db tools ?test_callback ?(systray=false)
           | `Select_only -> on_success ()
           | `Download_only | `Select_for_run ->
               let sels = Zeroinstall.Solver.selections results in
-              match_lwt Driver.download_selections config tools#distro (lazy fetcher) ~include_packages:true ~feed_provider:watcher#feed_provider sels with
+              Driver.download_selections config tools#distro (lazy fetcher) ~include_packages:true ~feed_provider:watcher#feed_provider sels >>= function
               | `aborted_by_user -> widgets.ok_button#set_active false; Lwt.return ()
               | `success -> on_success ()
         with Safe_exception _ as ex ->
@@ -248,34 +248,34 @@ let run_solver ~show_preferences ~trust_db tools ?test_callback ?(systray=false)
 
   let box_open_time = Unix.gettimeofday () in
 
-  (* Run a solve-with-downloads immediately, and every time the user clicks Refresh. *)
-  let refresh_loop =
-    while_lwt Lwt.state user_response = Lwt.Sleep do
-      need_recalculate := Lwt.wait ();
-      widgets.refresh_button#misc#set_sensitive false;
-      let force = !refresh in
-      refresh := false;
-      lwt (ready, _, _) = Driver.solve_with_downloads config tools#distro fetcher ~watcher reqs ~force ~update_local:true in
-      if Unix.gettimeofday () < box_open_time +. 1. then widgets.ok_button#grab_default ();
-      component_tree#highlight_problems;
+  let result = lazy (
+    (* Run a solve-with-downloads immediately, and every time the user clicks Refresh. *)
+    let refresh_loop =
+      while_lwt Lwt.state user_response = Lwt.Sleep do
+        need_recalculate := Lwt.wait ();
+        widgets.refresh_button#misc#set_sensitive false;
+        let force = !refresh in
+        refresh := false;
+        lwt (ready, _, _) = Driver.solve_with_downloads config tools#distro fetcher ~watcher reqs ~force ~update_local:true in
+        if Unix.gettimeofday () < box_open_time +. 1. then widgets.ok_button#grab_default ();
+        component_tree#highlight_problems;
 
-      if widgets.systray_icon#have_icon then (
-        if ready then (
-          widgets.systray_icon#set_tooltip (Printf.sprintf "Downloading updates for %s" reqs.Requirements.interface_uri);
-          widgets.ok_button#set_active true
-        ) else (
-          (* Should already be reporting an error, but blink it again just in case *)
-          widgets.systray_icon#set_blinking None
-        )
-      );
+        if widgets.systray_icon#have_icon then (
+          if ready then (
+            widgets.systray_icon#set_tooltip (Printf.sprintf "Downloading updates for %s" reqs.Requirements.interface_uri);
+            widgets.ok_button#set_active true
+          ) else (
+            (* Should already be reporting an error, but blink it again just in case *)
+            widgets.systray_icon#set_blinking None
+          )
+        );
 
-      (* Wait for user choice or refresh request *)
-      widgets.refresh_button#misc#set_sensitive true;
-      component_tree#set_update_icons true;
-      fst !need_recalculate
-    done in
+        (* Wait for user choice or refresh request *)
+        widgets.refresh_button#misc#set_sensitive true;
+        component_tree#set_update_icons true;
+        fst !need_recalculate
+      done in
 
-  let result =
     (* Wait for user to click Cancel or Run *)
     lwt response = user_response in
     watcher#abort_all_downloads;
@@ -288,11 +288,12 @@ let run_solver ~show_preferences ~trust_db tools ?test_callback ?(systray=false)
         let (ready, results) = watcher#results in
         assert ready;
         `Success (Zeroinstall.Solver.selections results) |> Lwt.return
-    | `aborted_by_user -> `Aborted_by_user |> Lwt.return in
+    | `aborted_by_user -> `Aborted_by_user |> Lwt.return
+  ) in
 
   object
     method recalculate = recalculate ~force:false
-    method result = result
+    method result = Lazy.force result
 
     (* Return the dialog window. If we're in systray mode, blink the icon and wait for the user
      * to click on it first. *)
@@ -347,11 +348,15 @@ let run_solver ~show_preferences ~trust_db tools ?test_callback ?(systray=false)
     method impl_added_to_store = component_tree#update
 
     method update =
-      let (ready, _) = watcher#results in
+      let (ready, results) = watcher#results in
       widgets.ok_button#misc#set_sensitive ready;
 
-      !component_boxes |> RoleMap.iter (fun _role box ->
-        box#update
+      let new_roles =
+        Zeroinstall.Solver.Model.raw_selections results
+        |> RoleMap.mapi (fun new_role _impl -> new_role) in
+
+      !component_boxes |> RoleMap.iter (fun old_role box ->
+        RoleMap.find old_role new_roles |> box#update
       );
 
       component_tree#update
