@@ -33,11 +33,16 @@ type other_req_option = [
   | `Source
 ]
 
-type select_option = [
+type binary_select_option = [
   | version_restriction_option
   | other_req_option
 ]
 
+type select_option = [
+  | binary_select_option
+  | `MayCompile
+]
+
 type generic_select_option = [
   | `Refresh
   | `ShowHuman
@@ -71,7 +76,7 @@ type tools = <
   trust_db : Zeroinstall.Trust.trust_db;
   set_use_gui : Support.Common.yes_no_maybe -> unit;
   use_gui : Support.Common.yes_no_maybe;
-  release : unit;     (** Call this to release any open connections held by the download pool. *)
+  release : unit;     (* Call this to release any open connections held by the download pool. *)
 >
 
 type global_settings = {
