@@ -48,7 +48,7 @@ let handle options flags args =
   let select_opts = ref [] in
   Support.Argparse.iter_options flags (function
     | #common_option as o -> Common_options.process_common_option options o
-    | #select_option | `Refresh as o -> select_opts := o :: !select_opts
+    | #binary_select_option | `Refresh as o -> select_opts := o :: !select_opts
     | `Wrapper w -> run_opts.wrapper <- Some w
     | `ShowManifest -> raise_safe "The -m argument is ambiguous before the 'run' argument. Put it after, or use --main"
     | `MainExecutable m -> run_opts.main <- Some m
