@@ -7,6 +7,7 @@
 open Options
 open Zeroinstall.General
 open Support.Common
+open Zeroinstall
 
 module J = Yojson.Basic
 module JC = Zeroinstall.Json_connection
@@ -31,7 +32,7 @@ let make_no_gui (connection:JC.json_connection) : Ui.ui_handler =
 
         let handle_pending fingerprint votes =
           let task =
-            lwt votes = votes in
+            votes >>= fun votes ->
             connection#invoke "update-key-info" [`String fingerprint; `List (json_of_votes votes)] in
           pending_tasks := task :: !pending_tasks in
 
@@ -44,7 +45,7 @@ let make_no_gui (connection:JC.json_connection) : Ui.ui_handler =
               | Lwt.Return votes -> json_of_votes votes in
             (fingerprint, `List json_votes)
           ) in
-          match_lwt connection#invoke "confirm-keys" [`String (Zeroinstall.Feed_url.format_url feed_url); `Assoc json_infos] with
+          connection#invoke "confirm-keys" [`String (Zeroinstall.Feed_url.format_url feed_url); `Assoc json_infos] >>= function
           | `List confirmed_keys -> confirmed_keys |> List.map Yojson.Basic.Util.to_string |> Lwt.return
           | _ -> raise_safe "Invalid response"
         finally
@@ -52,7 +53,7 @@ let make_no_gui (connection:JC.json_connection) : Ui.ui_handler =
           Lwt.return ()
 
       method! confirm message =
-        match_lwt connection#invoke "confirm" [`String message] with
+        connection#invoke "confirm" [`String message] >>= function
         | `String "ok" -> Lwt.return `ok
         | `String "cancel" -> Lwt.return `cancel
         | json -> raise_safe "Invalid response '%s'" (J.to_string json)
@@ -97,9 +98,10 @@ let parse_requirements json_assoc =
       interface_uri = pop "interface" |> J.Util.to_string;
       command = pop "command" |> J.Util.to_string_option;
       source = pop "source" |> J.Util.to_bool_option |> default false;
+      may_compile = pop "may_compile" |> J.Util.to_bool_option |> default false;
       extra_restrictions = pop "extra_restrictions" |> parse_restrictions;
-      os = pop "os" |> J.Util.to_string_option;
-      cpu = pop "cpu" |> J.Util.to_string_option;
+      os = pop "os" |> J.Util.to_string_option |> pipe_some Arch.parse_os;
+      cpu = pop "cpu" |> J.Util.to_string_option |> pipe_some Arch.parse_machine;
       message = pop "message" |> J.Util.to_string_option;
   }) in
   table |> Hashtbl.iter (fun name _ -> log_warning "Unexpected requirements field '%s'!" name);
@@ -114,7 +116,7 @@ let select options (ui:Zeroinstall.Ui.ui_handler) requirements refresh =
     `WithXML (json, Zeroinstall.Selections.as_xml sels) |> Lwt.return in
 
   let select_with_refresh refresh =
-    match_lwt ui#run_solver options.tools `Select_only requirements ~refresh with
+    ui#run_solver options.tools `Select_only requirements ~refresh >>= function
     | `Success sels -> success ~sels ~stale:false
     | `Aborted_by_user -> `List [`String "aborted-by-user"] |> Lwt.return in
 
@@ -143,9 +145,9 @@ let wrap_for_2_6 next (op, args) =
   match op with
   | "select" ->
       (* Strip out the new "status" response for old clients *)
-      begin match_lwt next (op, args) with
-      | `WithXML (`List [`String "ok"; _info], xml) -> `WithXML (`List [`String "ok"], xml) |> Lwt.return
-      | x -> Lwt.return x end;
+      begin next (op, args) >|= function
+      | `WithXML (`List [`String "ok"; _info], xml) -> `WithXML (`List [`String "ok"], xml)
+      | x -> x end;
   | _ -> next (op, args)
 
 let handle options flags args =
