@@ -160,6 +160,10 @@ let pipe_some fn = function
   | None -> None
   | Some x -> fn x
 
+let map_some fn = function
+  | None -> None
+  | Some x -> Some (fn x)
+
 (** {2 Backported from OCaml 4} **)
 
 let trim s =
@@ -182,3 +186,7 @@ let trim s =
     sub s !i (!j - !i + 1)
   else
     ""
+
+let (>>=) = Lwt.(>>=)
+let (>|=) = Lwt.(>|=)
+let return = Lwt.return
