@@ -243,7 +243,9 @@ let check_output ?env ?stderr ?reaper (system:system) fn (argv:string list) =
           let () =
             try
               system#reap_child ~kill_first:Sys.sigterm child_pid
-            with ex2 -> log_warning ~ex:ex2 "reap_child failed" in
+            with ex2 ->
+              let args = Logging.format_argv_for_logging argv in
+              log_warning ~ex:ex2 "reap_child failed (%s)" args in
           raise ex
       )
     in
@@ -557,3 +559,12 @@ let with_switch fn =
   Lwt.finalize
     (fun () -> fn switch)
     (fun () -> Lwt_switch.turn_off switch)
+
+let memoize ~initial_size create =
+  let cache = Hashtbl.create initial_size in
+  fun key ->
+    try Hashtbl.find cache key
+    with Not_found ->
+      let value = create key in
+      Hashtbl.add cache key value;
+      value
