@@ -82,10 +82,42 @@ let capture_stdout ?(include_stderr=false) fn =
         )
     )
 
+let capture_stdout_lwt ?(include_stderr=false) fn =
+  let open Unix in
+  let old_stdout = dup Unix.stdout in
+  let old_stderr = dup Unix.stderr in
+  try_lwt
+    let tmp = Filename.temp_file "0install-" "-test-output" in
+    let tmpfd = openfile tmp [O_RDWR] 0o600 in
+    try_lwt
+      dup2 tmpfd Unix.stdout;
+      if include_stderr then dup2 tmpfd Unix.stderr;
+      lwt () = fn () in
+      flush_all ();
+      U.read_file real_system tmp |> Lwt.return
+    finally
+      flush_all ();
+      close tmpfd;
+      unlink tmp;
+      Lwt.return ()
+  finally
+    dup2 old_stdout Unix.stdout; close old_stdout;
+    dup2 old_stderr Unix.stderr; close old_stderr;
+    Lwt.return ()
+
 exception Did_exec
 exception Would_exec of (bool * string array option * string list)
 exception Would_spawn of (bool * string array option * string list)
 
+let () =
+  Printexc.register_printer (function
+    | Would_exec (_search, _env, argv) ->
+        Some (Printf.sprintf "Test tried to exec process %s" (Support.Logging.format_argv_for_logging argv))
+    | Would_spawn (_search, _env, argv) ->
+        Some (Printf.sprintf "Test tried to spawn process %s" (Support.Logging.format_argv_for_logging argv))
+    | _ -> None
+  )
+
 let ocaml_dir = Sys.getcwd ()
 let src_dir = Filename.dirname ocaml_dir
 let tests_dir = ocaml_dir +/ "tests"
@@ -168,7 +200,12 @@ class fake_system tmpdir =
     method with_open_in flags fn path = real_system#with_open_in flags fn (check_read path)
     method with_open_out flags ~mode fn path = real_system#with_open_out flags ~mode fn (check_write path)
 
-    method mkdir path mode = real_system#mkdir (check_write path) mode
+    method mkdir path mode =
+      let path = check_write path in
+      let parent = Filename.dirname path in
+      if not (real_system#file_exists parent) && StringMap.mem parent !extra_files then
+        U.makedirs real_system parent 0o700;
+      real_system#mkdir path mode
 
     method rename source target =
       device_boundary |> if_some (fun device_boundary ->
@@ -227,7 +264,8 @@ class fake_system tmpdir =
           | Dir (mode, _items) -> Some (make_stat mode S_DIR)
           | File (_mode, target) -> real_system#stat target
         with Not_found ->
-          if hidden_subtree path then None
+          if tmpdir = None then None
+          else if hidden_subtree path then None
           else real_system#stat (check_read path)
       )
 
@@ -436,6 +474,14 @@ let collect_logging fn =
   forward_to_real_log := false;
   U.finally_do (fun () -> forward_to_real_log := true) () fn
 
+let collect_logging_lwt fn =
+  forward_to_real_log := false;
+  try_lwt
+    fn ()
+  finally
+    forward_to_real_log := true;
+    Lwt.return ()
+
 let format_list l = "[" ^ (String.concat "; " l) ^ "]"
 let equal_str_lists = assert_equal ~printer:format_list
 let assert_str_equal = assert_equal ~printer:(fun x -> x)
@@ -446,6 +492,16 @@ let assert_raises_safe expected_msg (fn:unit Lazy.t) =
     if not (Str.string_match (Str.regexp expected_msg) msg 0) then
       raise_safe "Error '%s' does not match regexp '%s'" msg expected_msg
 
+let assert_raises_safe_lwt expected_msg fn =
+  try_lwt
+    lwt () = fn () in
+    assert_failure ("Expected Safe_exception " ^ expected_msg)
+  with Safe_exception (msg, _) ->
+    if Str.string_match (Str.regexp expected_msg) msg 0 then
+      Lwt.return ()
+    else
+      raise_safe "Error '%s' does not match regexp '%s'" msg expected_msg
+
 let temp_dir_name =
   (* Filename.get_temp_dir_name doesn't exist under 3.12 *)
   U.realpath real_system
@@ -460,7 +516,12 @@ let with_tmpdir fn () =
     (fun () -> Unix.putenv "ZEROINSTALL_PORTABLE_BASE" "/UNUSED"; Unix.putenv "GNUPGHOME" "/UNUSED") ()
     (fun () ->
       let tmppath = U.make_tmp_dir real_system temp_dir_name ~prefix:"0install-test-" in
-      U.finally_do (U.rmtree ~even_if_locked:true real_system) tmppath fn
+      U.finally_do
+        (fun tmppath ->
+          try U.rmtree ~even_if_locked:true real_system tmppath
+          with ex -> log_warning ~ex "Failed to delete tmpdir!" (* Don't hide underlying error *)
+        )
+        tmppath fn
     )
 
 let get_fake_config tmpdir =
@@ -506,12 +567,12 @@ let assert_error_contains expected (fn:unit -> unit) =
   with Safe_exception (msg, _) ->
     assert_contains expected msg
 
-let fake_packagekit _config =
+let fake_packagekit status _config =
   object (_ : Zeroinstall.Packagekit.packagekit)
-    method is_available = Lwt.return false
-    method get_impls (package_name:string) : Zeroinstall.Packagekit.package_info list =
+    method status = Lwt.return status
+    method get_impls (package_name:string) =
       log_info "packagekit: get_impls(%s)" package_name;
-      []
+      { Zeroinstall.Packagekit.results = []; problems = [] }
     method check_for_candidates ~ui:_ ~hint (package_names:string list) : unit Lwt.t =
       log_info "packagekit: check_for_candidates(%s) for %s" (String.concat ", " package_names) hint;
       Lwt.return ()
