@@ -275,7 +275,7 @@ let show_log_on_failure fn () =
   Zeroinstall.Downloader.interceptor := None;
   Fake_system.forward_to_real_log := true;
   Update.wait_for_network := (fun () -> `Connected);
-  Zeroinstall.Packagekit.packagekit := Fake_system.fake_packagekit;
+  Zeroinstall.Packagekit.packagekit := Fake_system.fake_packagekit `Ok;
   try
     Fake_system.fake_log#reset;
     fn ();
@@ -300,7 +300,12 @@ let () =
   )
 ENDIF
 
+let is_error = function
+  | RFailure _ | RError _ -> true
+  | _ -> false
+
 let () =
   Printexc.record_backtrace true;
-  ignore @@ run_test_tt_main @@ test_decorate show_log_on_failure suite;
-  Format.print_newline ()
+  let results = run_test_tt_main @@ test_decorate show_log_on_failure suite in
+  Format.print_newline ();
+  if List.exists is_error results then exit 1
