@@ -14,6 +14,16 @@ module Impl = Zeroinstall.Impl
 module F = Zeroinstall.Feed
 module U = Support.Utils
 
+let is_available_locally config impl =
+  let open Impl in
+  match impl.impl_type with
+  | `package_impl {package_state;_} -> package_state = `installed
+  | `local_impl path -> config.system#file_exists path
+  | `cache_impl {digests;_} ->
+      match Zeroinstall.Stores.lookup_maybe config.system digests config.stores with
+      | None -> false
+      | Some _path -> true
+
 let test_feed = "<?xml version='1.0'?>\n\
 <interface xmlns='http://zero-install.sourceforge.net/2004/injector/interface' uri='http://repo.roscidus.com/python/python'>\n\
   <name>Test</name>\n\
@@ -73,13 +83,13 @@ let suite = "distro">::: [
     let system = (fake_system :> system) in
     let distro = Distro_impls.ArchLinux.arch_distribution config in
     let feed = load_feed system test_feed in
-    let impls = distro#get_impls_for_feed feed |> to_impl_list in
+    let impls = distro#get_impls_for_feed ~problem:failwith feed |> to_impl_list in
     let open Impl in
     match impls with
     | [impl] ->
         assert_str_equal "2.7.2-4" (Zeroinstall.Version.to_string impl.parsed_version);
         let run = StringMap.find_safe "run" impl.props.commands in
-        assert_str_equal "/bin/python2" (ZI.get_attribute "path" run.command_qdom)
+        assert_str_equal "/bin/python2" (Element.path run.command_qdom |> Fake_system.expect)
     | impls -> assert_failure @@ Printf.sprintf "want 1 Python, got %d" (List.length impls)
   );
 
@@ -88,9 +98,9 @@ let suite = "distro">::: [
     let arch_db = Test_0install.feed_dir +/ "arch" in
     let distro = Distro_impls.ArchLinux.arch_distribution ~arch_db config in
 
-    distro#get_impls_for_feed gimp_feed |> to_impl_list |> assert_equal [];
+    distro#get_impls_for_feed ~problem:failwith gimp_feed |> to_impl_list |> assert_equal [];
 
-    begin match distro#get_impls_for_feed (make_test_feed "zeroinstall-injector") |> to_impl_list with
+    begin match distro#get_impls_for_feed ~problem:failwith (make_test_feed "zeroinstall-injector") |> to_impl_list with
     | [impl] ->
         assert_str_equal "package:arch:zeroinstall-injector:1.5-1:*" @@ Impl.get_attr_ex "id" impl;
         assert_str_equal "1.5-1" @@ Impl.get_attr_ex "version" impl
@@ -103,13 +113,13 @@ let suite = "distro">::: [
     let packages_dir = slackdir +/ "packages" in
     let distro = Distro_impls.Slackware.slack_distribution ~packages_dir config in
 
-    distro#get_impls_for_feed gimp_feed |> to_impl_list |> assert_equal [];
+    distro#get_impls_for_feed ~problem:failwith gimp_feed |> to_impl_list |> assert_equal [];
 
-    begin match distro#get_impls_for_feed (make_test_feed "infozip") |> to_impl_list with
+    begin match distro#get_impls_for_feed ~problem:failwith (make_test_feed "infozip") |> to_impl_list with
     | [impl] ->
         assert_str_equal "package:slack:infozip:5.52-2:i486" @@ Impl.get_attr_ex "id" impl;
         assert_str_equal "5.52-2" @@ Impl.get_attr_ex "version" impl;
-        assert_str_equal "i486" @@ (expect impl.Impl.machine);
+        assert_str_equal "i486" @@ Arch.format_machine_or_star impl.Impl.machine;
     | impls -> assert_failure @@ Printf.sprintf "want 1, got %d" (List.length impls) end;
   );
 
@@ -118,31 +128,31 @@ let suite = "distro">::: [
     let pkgdir = Test_0install.feed_dir +/ "gentoo" in
     let distro = Distro_impls.Gentoo.gentoo_distribution ~pkgdir config in
 
-    distro#get_impls_for_feed gimp_feed |> to_impl_list |> assert_equal [];
+    distro#get_impls_for_feed ~problem:failwith gimp_feed |> to_impl_list |> assert_equal [];
 
-    begin match distro#get_impls_for_feed (make_test_feed "sys-apps/portage") |> to_impl_list with
+    begin match distro#get_impls_for_feed ~problem:failwith (make_test_feed "sys-apps/portage") |> to_impl_list with
     | [impl] ->
         assert_str_equal "package:gentoo:sys-apps/portage:2.1.7.16:x86_64" @@ Impl.get_attr_ex "id" impl;
         assert_str_equal "2.1.7.16" @@ Impl.get_attr_ex "version" impl;
-        assert_str_equal "x86_64" @@ (expect impl.Impl.machine);
+        assert_str_equal "x86_64" @@ Arch.format_machine_or_star impl.Impl.machine;
     | impls -> assert_failure @@ Printf.sprintf "want 1, got %d" (List.length impls) end;
 
-    begin match distro#get_impls_for_feed (make_test_feed "sys-kernel/gentoo-sources") |> to_impl_list with
+    begin match distro#get_impls_for_feed ~problem:failwith (make_test_feed "sys-kernel/gentoo-sources") |> to_impl_list with
     | [b; a] ->
         assert_str_equal "package:gentoo:sys-kernel/gentoo-sources:2.6.30-4:i686" @@ Impl.get_attr_ex "id" a;
         assert_str_equal "2.6.30-4" @@ Impl.get_attr_ex "version" a;
-        assert_str_equal "i686" @@ (expect a.Impl.machine);
+        assert_str_equal "i686" @@ Arch.format_machine_or_star a.Impl.machine;
 
         assert_str_equal "package:gentoo:sys-kernel/gentoo-sources:2.6.32:x86_64" @@ Impl.get_attr_ex "id" b;
         assert_str_equal "2.6.32" @@ Impl.get_attr_ex "version" b;
-        assert_str_equal "x86_64" @@ (expect b.Impl.machine);
+        assert_str_equal "x86_64" @@ Arch.format_machine_or_star b.Impl.machine;
     | impls -> assert_failure @@ Printf.sprintf "want 2, got %d" (List.length impls) end;
 
-    begin match distro#get_impls_for_feed (make_test_feed "app-emulation/emul-linux-x86-baselibs") |> to_impl_list with
+    begin match distro#get_impls_for_feed ~problem:failwith (make_test_feed "app-emulation/emul-linux-x86-baselibs") |> to_impl_list with
     | [impl] ->
         assert_str_equal "package:gentoo:app-emulation/emul-linux-x86-baselibs:20100220:i386" @@ Impl.get_attr_ex "id" impl;
         assert_str_equal "20100220" @@ Impl.get_attr_ex "version" impl;
-        assert_str_equal "i386" @@ (expect impl.Impl.machine);
+        assert_str_equal "i386" @@ Arch.format_machine_or_star impl.Impl.machine;
     | impls -> assert_failure @@ Printf.sprintf "want 1, got %d" (List.length impls) end;
   );
 
@@ -151,7 +161,7 @@ let suite = "distro">::: [
     let pkg_db = Test_0install.feed_dir +/ "ports" in
     let distro = Distro_impls.Ports.ports_distribution ~pkg_db config in
 
-    begin match distro#get_impls_for_feed (make_test_feed "zeroinstall-injector") |> to_impl_list with
+    begin match distro#get_impls_for_feed ~problem:failwith (make_test_feed "zeroinstall-injector") |> to_impl_list with
     | [impl] ->
         assert (U.starts_with (Impl.get_attr_ex "id" impl) "package:ports:zeroinstall-injector:0.41-2:");
         assert_str_equal "0.41-2" @@ Impl.get_attr_ex "version" impl
@@ -168,7 +178,7 @@ let suite = "distro">::: [
     let macports_db = pkgdir +/ "registry.db" in
     let distro = Distro_impls.Mac.macports_distribution ~macports_db config in
 
-    begin match distro#get_impls_for_feed (make_test_feed "zeroinstall-injector") |> to_impl_list with
+    begin match distro#get_impls_for_feed ~problem:failwith (make_test_feed "zeroinstall-injector") |> to_impl_list with
     | [impl] ->
         assert_str_equal "package:macports:zeroinstall-injector:1.0-0:*" @@ Impl.get_attr_ex "id" impl;
         assert_str_equal "1.0-0" @@ Impl.get_attr_ex "version" impl;
@@ -200,18 +210,18 @@ let suite = "distro">::: [
     let root = Q.parse_input None @@ Xmlm.make_input (`String (0, test_feed)) |> Element.parse_feed in
     let feed = F.parse system root None in
     let () =
-      let impls = distro#get_impls_for_feed feed in
+      let impls = distro#get_impls_for_feed ~problem:failwith feed in
       let host_python = find_host impls in
       let python_run =
         try StringMap.find_nf "run" host_python.props.commands
         with Not_found -> assert_failure "No run command for host Python" in
-      assert (Fake_system.real_system#file_exists (ZI.get_attribute "path" python_run.command_qdom)) in
+      assert (Fake_system.real_system#file_exists (Element.path python_run.command_qdom |> Fake_system.expect)) in
 
     (* python-gobject *)
     let root = Q.parse_input None @@ Xmlm.make_input (`String (0, test_gobject_feed)) |> Element.parse_feed in
     let feed = F.parse system root None in
 
-    let impls = distro#get_impls_for_feed feed in
+    let impls = distro#get_impls_for_feed ~problem:failwith feed in
     let host_gobject =
       try impls |> StringMap.bindings |> List.find is_host |> snd
       with Not_found -> skip_if true "No host python-gobject found"; assert false in
@@ -227,7 +237,7 @@ let suite = "distro">::: [
       ~attrs:(Q.AttrMap.singleton "interface" "http://repo.roscidus.com/python/python")
       ~child_nodes:[sel] |> Element.parse_selections in
     match Element.selections sels with
-    | [sel] -> assert (Distro.is_installed config distro sel)
+    | [sel] -> assert (distro#is_installed_quick sel)
     | _ -> assert false
   );
 
@@ -249,36 +259,36 @@ let suite = "distro">::: [
     let feed = get_feed
       "<package-implementation distributions='Debian' package='yast2-mail'/>\n\
        <package-implementation distributions='RPM' package='yast2-update'/>" in
-    let impls = to_impl_list @@ rpm#get_impls_for_feed feed in
+    let impls = to_impl_list @@ rpm#get_impls_for_feed ~problem:failwith feed in
     begin match impls with
     | [yast] ->
         assert_equal "package:rpm:yast2-update:2.15.23-21:i586" (Impl.get_attr_ex "id" yast);
         assert_equal "2.15.23-21" (Impl.get_attr_ex "version" yast);
-        assert_equal "*-i586" (Zeroinstall.Arch.format_arch yast.Impl.os yast.Impl.machine);
+        assert_equal "*-i586" (Zeroinstall.Arch.format_arch (yast.Impl.os, yast.Impl.machine));
     | _ -> assert false end;
 
     let feed = get_feed "<package-implementation distributions='RPM' package='yast2-mail'/>\n\
                          <package-implementation distributions='RPM' package='yast2-update'/>" in
-    let impls = to_impl_list @@ rpm#get_impls_for_feed feed in
+    let impls = to_impl_list @@ rpm#get_impls_for_feed ~problem:failwith feed in
     assert_equal 2 (List.length impls);
 
     let feed = get_feed "<package-implementation distributions='' package='yast2-mail'/>\n\
                          <package-implementation package='yast2-update'/>" in
-    let impls = to_impl_list @@ rpm#get_impls_for_feed feed in
+    let impls = to_impl_list @@ rpm#get_impls_for_feed ~problem:failwith feed in
     assert_equal 2 (List.length impls);
 
     let feed = get_feed "<package-implementation distributions='Foo Bar Baz' package='yast2-mail'/>" in
-    let impls = to_impl_list @@ rpm#get_impls_for_feed feed in
+    let impls = to_impl_list @@ rpm#get_impls_for_feed ~problem:failwith feed in
     assert_equal 1 (List.length impls);
 
     (* Check the caching worked *)
     fake_system#set_spawn_handler None;
-    let impls = to_impl_list @@ rpm#get_impls_for_feed feed in
+    let impls = to_impl_list @@ rpm#get_impls_for_feed ~problem:failwith feed in
     assert_equal 1 (List.length impls);
 
     (* Check escaping works *)
     let feed = get_feed "<package-implementation package='poor=name'/>" in
-    let impls = to_impl_list @@ rpm#get_impls_for_feed feed in
+    let impls = to_impl_list @@ rpm#get_impls_for_feed ~problem:failwith feed in
     assert_equal 1 (List.length impls);
 
     Unix.putenv "PATH" old_path;
@@ -286,6 +296,7 @@ let suite = "distro">::: [
 
   "debian">:: Fake_system.with_fake_config (fun (config, fake_system) ->
     skip_if (Sys.os_type = "Win32") "Paths get messed up on Windows";
+    Zeroinstall.Packagekit.packagekit := Fake_system.fake_packagekit (`Unavailable "Use apt-cache");
     let xml =
       "<?xml version='1.0' ?>\n\
       <interface xmlns='http://zero-install.sourceforge.net/2004/injector/interface'>\n\
@@ -307,13 +318,13 @@ let suite = "distro">::: [
     fake_system#putenv "PATH" (dpkgdir ^ ":" ^ old_path);
     fake_system#set_spawn_handler (Some Fake_system.real_spawn_handler);
     let deb = Distro_impls.Debian.debian_distribution ~status_file:(dpkgdir +/ "status") config in
-    begin match to_impl_list @@ deb#get_impls_for_feed feed with
+    begin match to_impl_list @@ deb#get_impls_for_feed ~problem:failwith feed with
     | [impl] ->
         Fake_system.assert_str_equal "package:deb:python-bittorrent:3.4.2-10-2:*" (Impl.get_attr_ex "id" impl);
         assert_equal ~msg:"Stability" Packaged impl.Impl.stability;
         assert_equal ~msg:"Requires" [] impl.Impl.props.Impl.requires;
-        Fake_system.assert_str_equal "/usr/bin/pbt" (ZI.get_attribute_opt "main" impl.Impl.qdom |> Fake_system.expect);
-        impl.Impl.qdom.Q.attrs |> Q.AttrMap.get_no_ns "foo" |> assert_equal (Some "bar");
+        Fake_system.assert_str_equal "/usr/bin/pbt" (Element.main impl.Impl.qdom |> Fake_system.expect);
+        (Element.as_xml impl.Impl.qdom).Q.attrs |> Q.AttrMap.get_no_ns "foo" |> assert_equal (Some "bar");
         Fake_system.assert_str_equal "distribution:/local.xml" (Impl.get_attr_ex "from-feed" impl);
     | _ -> assert false end;
 
@@ -347,7 +358,7 @@ let suite = "distro">::: [
 
     (* Part II *)
     let gimp_feed = get_feed "<package-implementation package='gimp'/>" in
-    deb#get_impls_for_feed gimp_feed |> assert_equal StringMap.empty;
+    deb#get_impls_for_feed ~problem:failwith gimp_feed |> assert_equal StringMap.empty;
 
     (* Initially, we only get information about the installed version... *)
     let bt_feed = get_feed "<package-implementation package='python-bittorrent'>\n\
@@ -355,7 +366,7 @@ let suite = "distro">::: [
                                   <version not-before='3'/>\n\
                                 </restricts>\n\
                                 </package-implementation>" in
-    deb#get_impls_for_feed bt_feed |> to_impl_list |> List.length |> assert_equal 1;
+    deb#get_impls_for_feed ~problem:failwith bt_feed |> to_impl_list |> List.length |> assert_equal 1;
 
 
     Fake_system.fake_log#reset;
@@ -365,7 +376,7 @@ let suite = "distro">::: [
 
     (* Now we see the uninstalled package *)
     let compare_version a b = compare a.Impl.parsed_version b.Impl.parsed_version in
-    begin match to_impl_list @@ deb#get_impls_for_feed bt_feed |> List.sort compare_version with
+    begin match to_impl_list @@ deb#get_impls_for_feed ~problem:failwith bt_feed |> List.sort compare_version with
     | [installed; uninstalled] as impls ->
         (* Check restriction appears for both candidates *)
         impls |> List.iter (fun impl ->
@@ -374,30 +385,30 @@ let suite = "distro">::: [
           | _ -> assert false
         );
         Fake_system.assert_str_equal "3.4.2-10-2" (Impl.get_attr_ex "version" installed);
-        assert_equal true @@ Impl.is_available_locally config installed;
-        assert_equal false @@ Impl.is_available_locally config uninstalled;
+        assert_equal true @@ is_available_locally config installed;
+        assert_equal false @@ is_available_locally config uninstalled;
         assert_equal None installed.Impl.machine;
     | _ -> assert false
     end;
 
     let feed = get_feed "<package-implementation package='libxcomposite-dev'/>" in
-    begin match to_impl_list @@ deb#get_impls_for_feed feed with
+    begin match to_impl_list @@ deb#get_impls_for_feed ~problem:failwith feed with
     | [libxcomposite] ->
         Fake_system.assert_str_equal "0.3.1-1" @@ Impl.get_attr_ex "version" libxcomposite;
-        Fake_system.assert_str_equal "i386" @@ Fake_system.expect libxcomposite.Impl.machine
+        Fake_system.assert_str_equal "i386" @@ Arch.format_machine_or_star libxcomposite.Impl.machine
     | _ -> assert false
     end;
 
     (* Java is special... *)
     let feed = get_feed "<package-implementation package='openjdk-7-jre'/>" in
-    begin match to_impl_list @@ deb#get_impls_for_feed feed with
+    begin match to_impl_list @@ deb#get_impls_for_feed ~problem:failwith feed with
     | [impl] -> Fake_system.assert_str_equal "7.3-2.1.1-3" @@ Impl.get_attr_ex "version" impl
     | _ -> assert false end;
 
     (* Check the disk cache works *)
     fake_system#set_spawn_handler None;
     let deb = Distro_impls.Debian.debian_distribution ~status_file:(dpkgdir +/ "status") config in
-    begin match to_impl_list @@ deb#get_impls_for_feed feed with
+    begin match to_impl_list @@ deb#get_impls_for_feed ~problem:failwith feed with
     | [impl] -> Fake_system.assert_str_equal "7.3-2.1.1-3" @@ Impl.get_attr_ex "version" impl
     | _ -> assert false end;
 
@@ -421,7 +432,7 @@ let suite = "distro">::: [
         val distro_name = "test"
         val id_prefix = "package:test"
 
-        method! private get_package_impls query =
+        method private get_package_impls query =
           match query.Distro.package_name with
           | "foo" ->
               self#add_package_implementation
@@ -433,16 +444,19 @@ let suite = "distro">::: [
                 ~distro_name
                 query
           | _ -> ()
+
+        method check_for_candidates ~ui:_ _feed = Lwt.return ()
       end in
     let feed_provider = new Test_solver.fake_feed_provider config.system (Some distro) in
     feed_provider#add_iface master_feed;
     feed_provider#add_iface imported_feed;
-    let scope_filter = { Impl_provider.
+    let scope_filter = { Scope_filter.
       extra_restrictions = StringMap.empty;
-      os_ranks = StringMap.empty;
-      machine_ranks = StringMap.empty;
+      os_ranks = Arch.custom_os_ranking StringMap.empty;
+      machine_ranks = Arch.custom_machine_ranking StringMap.empty;
       languages = Support.Locale.LangMap.empty;
       allowed_uses = StringSet.empty;
+      may_compile = false;
     } in
     let impl_provider = new Impl_provider.default_impl_provider config (feed_provider :> Feed_provider.feed_provider) scope_filter in
     let impls = impl_provider#get_implementations master_uri ~source:false in
