@@ -53,8 +53,7 @@ let fake_gui =
   end
 
 let install_interceptor system checked_for_gui =
-  Zeroinstall.Gui.register_plugin (fun _config use_gui ->
-    assert (use_gui = Maybe);
+  Zeroinstall.Gui.register_plugin (fun _config ->
     let have_gui = system#getenv "DISPLAY" <> Some "" in
     checked_for_gui := true;
     if have_gui then (
@@ -279,6 +278,7 @@ let suite = "download">::: [
   );
 
   "distro">:: Server.with_server (fun (config, fake_system) server ->
+    Zeroinstall.Packagekit.packagekit := Fake_system.fake_packagekit (`Unavailable "Use apt-cache");
     let native_url = "http://example.com:8000/Native.xml" in
     fake_system#set_spawn_handler (Some Fake_system.real_spawn_handler);
 
@@ -305,7 +305,7 @@ let suite = "download">::: [
 
     Fake_system.fake_log#reset;
     Lwt_main.run @@ deb#check_for_candidates ~ui:Fake_system.null_ui feed;
-    begin match Test_distro.to_impl_list @@ deb#get_impls_for_feed feed with
+    begin match Test_distro.to_impl_list @@ deb#get_impls_for_feed ~problem:failwith feed with
     | [_impl1; _impl2] -> ()
     | items -> raise_safe "Got %d!" (List.length items) end;
 
@@ -593,6 +593,7 @@ let suite = "download">::: [
 
   "background-unsolvable">:: Server.with_server (fun (config, fake_system) server ->
     fake_system#allow_spawn_detach true;
+    fake_system#set_spawn_handler (Some Fake_system.real_spawn_handler);  (* For "ports -v" *)
     let trust_db = new Zeroinstall.Trust.trust_db config in
     let system = config.system in
     let home = U.getenv_ex system "HOME" in
