@@ -22,7 +22,7 @@ module S = Support.Sat.MakeSAT(StringData)
 let re_comma = Str.regexp_string ","
 
 class impl_provider =
-  let dummy_element = General.ZI.make "implementation" in
+  let dummy_element = Element.make_impl Q.AttrMap.empty in
 
   let make_impl arch v =
     let (os, machine) = Arch.parse_arch arch in
@@ -62,6 +62,7 @@ class impl_provider =
       replacement = None;
       rejects = [];
       compare = (fun _ _ -> failwith "compare");
+      feed_problems = [];
       impls =
         try Hashtbl.find interfaces iface
         with Not_found -> failwith iface;
