@@ -105,7 +105,7 @@ let handle options flags args =
   let select_opts = ref [] in
   Support.Argparse.iter_options flags (function
     | #common_option as o -> Common_options.process_common_option options o
-    | #select_option as o -> select_opts := o :: !select_opts
+    | #binary_select_option as o -> select_opts := o :: !select_opts
     | `Refresh -> log_warning "deprecated: update implies --refresh anyway"
   );
   match args with
@@ -135,9 +135,9 @@ let handle options flags args =
 let notify = ref (fun ~msg ~timeout ->
   Lwt_main.run (
     try_lwt
-      match_lwt D.session () with
-      | None -> log_info "0install: %s" msg; Lwt.return ()
-      | Some _bus ->
+      D.session () >>= function
+      | `Error _ -> log_info "0install: %s" msg; Lwt.return ()
+      | `Ok _bus ->
           ignore (D.Notification.notify ~timeout ~summary:"0install" ~body:msg ~icon:"info" ());
 
           (* Force a round-trip to make sure the notice has been sent before we exit
@@ -153,10 +153,10 @@ let notify = ref (fun ~msg ~timeout ->
 let get_network_state () : D.Nm_manager.state =
   Lwt_main.run (
     try_lwt
-      match_lwt D.system () with
-      | None -> Lwt.return `Unknown
-      | Some _bus ->
-          lwt daemon = D.Nm_manager.daemon () in
+      D.system () >>= function
+      | `Error _ -> Lwt.return `Unknown
+      | `Ok _bus ->
+          D.Nm_manager.daemon () >>= fun daemon ->
           D.OBus_property.get (D.Nm_manager.state daemon)
     with ex ->
       log_info ~ex "Failed to get NetworkManager state";
