@@ -5,7 +5,6 @@
 (** Interacting with distribution package managers. *)
 
 open General
-open Support
 open Support.Common
 module FeedAttr = Constants.FeedAttr
 module U = Support.Utils
@@ -42,14 +41,16 @@ type query = {
   elem_props : Impl.properties;     (* Properties on or inherited by the <package-element> - used by [add_package_implementation] *)
   feed : Feed.feed;                 (* The feed containing the <package-element> *)
   results : Impl.distro_implementation Support.Common.StringMap.t ref;
+  problem : string -> unit;
 }
 
-let make_query feed elem elem_props results = {
+let make_query feed elem elem_props results problem = {
   elem;
   package_name = Element.package elem;
   elem_props;
   feed;
   results;
+  problem;
 }
 
 type quick_test_condition = Exists | UnchangedSince of float
@@ -118,7 +119,7 @@ class virtual distribution config =
   ) in
 
   let make_host_impl path version ~package ?(commands=StringMap.empty) ?(requires=[]) from_feed id =
-    let host_machine = system#platform in
+    let (_host_os, host_machine) = Arch.platform system in
     let props = { Impl.
       attrs = get_quick_test_attrs path
         |> Q.AttrMap.add_no_ns FeedAttr.from_feed (Feed_url.format_url (`distribution_feed from_feed))
@@ -130,11 +131,11 @@ class virtual distribution config =
       bindings = [];
       commands;
     } in { Impl.
-      qdom = ZI.make "host-package-implementation";
+      qdom = Element.make_impl Q.AttrMap.empty;
       props;
       stability = Packaged;
       os = None;
-      machine = Some host_machine.Platform.machine;       (* (hopefully) *)
+      machine = Some host_machine;       (* (hopefully) *)
       parsed_version = Version.parse version;
       impl_type = `package_impl { Impl.
         package_distro = "host";
@@ -149,12 +150,8 @@ class virtual distribution config =
            cache this information on disk. *)
         Lazy.force python_info |> List.map (fun ((path, version), _) ->
           let id = "package:host:python:" ^ version in
-          let run = ZI.make "command"
-            ~attrs:(
-              Q.AttrMap.singleton "name" "run"
-              |> Q.AttrMap.add_no_ns "path" path
-            ) in
-          let commands = StringMap.singleton "run" Impl.({command_qdom = run; command_requires = []; command_bindings = []}) in
+          let run = Impl.make_command "run" path in
+          let commands = StringMap.singleton "run" run in
           (id, make_host_impl ~package:"host-python" path version ~commands url id)
         )
     | `remote_feed "http://repo.roscidus.com/python/python-gobject" as url ->
@@ -167,18 +164,6 @@ class virtual distribution config =
         )
     | _ -> [] in
 
-  let fixup_main distro_get_correct_main impl =
-    let open Impl in
-    match get_command_opt "run" impl with
-    | None -> ()
-    | Some run ->
-        match distro_get_correct_main impl run with
-        | None -> ()
-        | Some new_main ->
-            run.command_qdom <- {run.command_qdom with
-              Q.attrs = run.command_qdom.Q.attrs |> Q.AttrMap.add_no_ns "path" new_main
-            } in
-
   object (self)
     val virtual distro_name : string
     val virtual check_host_python : bool
@@ -186,8 +171,6 @@ class virtual distribution config =
 
     val valid_package_name = Str.regexp "^[^.-][^/]*$"
 
-    val packagekit = !Packagekit.packagekit config
-
     (** All IDs will start with this string (e.g. "package:deb") *)
     val virtual id_prefix : string
 
@@ -201,10 +184,19 @@ class virtual distribution config =
     (** Can we use packages for this distribution? For example, MacPortsDistribution can use "MacPorts" and "Darwin" packages. *)
     method match_name name = (name = distro_name)
 
+    method private fixup_main impl =
+      let open Impl in
+      match get_command_opt "run" impl with
+      | None -> ()
+      | Some run ->
+          match self#get_correct_main impl run with
+          | None -> ()
+          | Some new_main -> run.command_qdom <- Element.make_command ~path:new_main ~source_hint:None "run"
+
     (** Convenience wrapper for [add_result] that builds a new implementation from the given attributes. *)
     method private add_package_implementation ?id ?main (query:query) ~version ~machine ~quick_test ~package_state ~distro_name =
       let version_str = Version.to_string version in
-      let id = id |? lazy (Printf.sprintf "%s:%s:%s:%s" id_prefix query.package_name version_str (default "*" machine)) in
+      let id = id |? lazy (Printf.sprintf "%s:%s:%s:%s" id_prefix query.package_name version_str (Arch.format_machine_or_star machine)) in
       let props = query.elem_props in
       let elem = query.elem in
 
@@ -217,11 +209,7 @@ class virtual distribution config =
             let run_command =
               match StringMap.find "run" props.commands with
               | Some command ->
-                  {command with command_qdom = 
-                    {command.command_qdom with
-                      Qdom.attrs = command.command_qdom.Qdom.attrs |> Qdom.AttrMap.add_no_ns "path" path
-                    }
-                  };
+                  {command with command_qdom = Element.make_command ~path ~source_hint:None "run"}
               | None ->
                   make_command "run" path in
             {props with commands = StringMap.add "run" run_command props.commands} in
@@ -244,7 +232,7 @@ class virtual distribution config =
 
       let open Impl in
       let impl = {
-        qdom = Element.as_xml elem;
+        qdom = (elem :> [ `implementation | `package_impl ] Element.t);
         os = None;
         machine;
         stability = Packaged;
@@ -253,14 +241,14 @@ class virtual distribution config =
         impl_type = `package_impl { package_state; package_distro = distro_name };
       } in
 
-      if package_state = `installed then fixup_main self#get_correct_main impl;
+      if package_state = `installed then self#fixup_main impl;
 
       query.results := StringMap.add id impl !(query.results)
 
     (** Test whether this <selection> element is still valid. The default implementation tries to load the feed from the
      * feed cache, calls [distribution#get_impls_for_feed] on it and checks whether the required implementation ID is in the
      * returned map. Override this if you can provide a more efficient implementation. *)
-    method is_installed elem =
+    method private is_installed elem =
       let master_feed =
         match Element.from_feed elem with
         | None -> Element.interface elem |> Feed_url.parse_non_distro (* (for very old selections documents) *)
@@ -272,14 +260,27 @@ class virtual distribution config =
       | None -> false
       | Some master_feed ->
           let wanted_id = Element.id elem in
-          let impls = self#get_impls_for_feed master_feed in
+          let impls = self#get_impls_for_feed ~problem:ignore master_feed in
           match StringMap.find wanted_id impls with
           | None -> false
           | Some {Impl.impl_type = `package_impl {Impl.package_state; _}; _} -> package_state = `installed
 
+    method is_installed_quick elem =
+      match Element.quick_test_file elem with
+      | None ->
+          let package_name = Element.package elem in
+          self#is_valid_package_name package_name && self#is_installed elem
+      | Some file ->
+          match system#stat file with
+          | None -> false
+          | Some info ->
+              match Element.quick_test_mtime elem with
+              | None -> true      (* quick-test-file exists and we don't care about the time *)
+              | Some required_mtime -> (Int64.of_float info.Unix.st_mtime) = required_mtime
+
     (** Get the native implementations (installed or candidates for installation) for this feed.
      * This default implementation finds the best <package-implementation> elements and calls [get_package_impls] on each one. *)
-    method get_impls_for_feed ?(init=StringMap.empty) (feed:Feed.feed) : Impl.distro_implementation StringMap.t =
+    method get_impls_for_feed ?(init=StringMap.empty) ~problem (feed:Feed.feed) : Impl.distro_implementation StringMap.t =
       let results = ref init in
 
       if check_host_python then (
@@ -290,32 +291,18 @@ class virtual distribution config =
       | [] -> !results
       | matches ->
           matches |> List.iter (fun (elem, props) ->
-            self#get_package_impls (make_query feed elem props results);
+            self#get_package_impls (make_query feed elem props results problem);
           );
           !results
 
-    method private get_package_impls query : unit =
-      let package_name = query.package_name in
-      packagekit#get_impls package_name |> List.iter (fun info ->
-        let {Packagekit.version; Packagekit.machine; Packagekit.installed; Packagekit.retrieval_method} = info in
-        let package_state =
-          if installed then `installed
-          else `uninstalled retrieval_method in
-        self#add_package_implementation
-          ~version
-          ~machine
-          ~package_state
-          ~quick_test:None
-          ~distro_name:distro_name
-          query
-      )
+    method virtual private get_package_impls : query -> unit
 
     (** Called when an installed package is added, or when installation completes. This is useful to fix up the main value.
         The default implementation checks that main exists, and searches [system_paths] for
         it if not. *)
-    method private get_correct_main _impl run_command =
+    method private get_correct_main (_impl:Impl.distro_implementation) run_command =
       let open Impl in
-      ZI.get_attribute_opt "path" run_command.command_qdom |> pipe_some (fun path ->
+      Element.path run_command.command_qdom |> pipe_some (fun path ->
         if Filename.is_relative path || not (system#file_exists path) then (
           (* Need to search for the binary *)
           let basename = Filename.basename path in
@@ -334,50 +321,19 @@ class virtual distribution config =
         ) else None
       )
 
-    (* This default implementation queries PackageKit, if available. *)
-    method check_for_candidates : 'a. ui:(#Packagekit.ui as 'a) -> Feed.feed -> unit Lwt.t = fun ~ui feed ->
-      match get_matching_package_impls self feed with
-      | [] -> Lwt.return ()
-      | matches ->
-          lwt available = packagekit#is_available in
-          if available then (
-            let package_names = matches |> List.map (fun (elem, _props) -> Element.package elem) in
-            let hint = Feed_url.format_url feed.Feed.url in
-            packagekit#check_for_candidates ~ui ~hint package_names
-          ) else Lwt.return ()
+    method virtual check_for_candidates : 'a. ui:(#Packagekit.ui as 'a) -> Feed.feed -> unit Lwt.t
 
-    method install_distro_packages : 'a. (#Packagekit.ui as 'a) -> string -> _ list -> [ `ok | `cancel ] Lwt.t =
+    method install_distro_packages : 'a. (#Packagekit.ui as 'a) -> string ->
+        (Impl.distro_implementation * Impl.distro_retrieval_method) list -> [ `ok | `cancel ] Lwt.t =
       fun ui typ items ->
-        match typ with
-        | "packagekit" ->
-            begin match_lwt packagekit#install_packages ui items with
-            | `cancel -> Lwt.return `cancel
-            | `ok ->
-                items |> List.iter (fun (impl, _rm) ->
-                  fixup_main self#get_correct_main impl
-                );
-                Lwt.return `ok end
-        | _ ->
-            let names = items |> List.map (fun (_impl, rm) -> snd rm.Impl.distro_install_info) in
-            ui#confirm (Printf.sprintf
-              "This program depends on some packages that are available through your distribution. \
-               Please install them manually using %s and try again. Or, install 'packagekit' and I can \
-               use that to install things. The packages are:\n\n- %s" typ (String.concat "\n- " names))
+        let names = items |> List.map (fun (_impl, rm) -> snd rm.Impl.distro_install_info) in
+        ui#confirm (Printf.sprintf
+          "This program depends on some packages that are available through your distribution. \
+           Please install them manually using %s before continuing. The packages are:\n\n- %s"
+           typ
+           (String.concat "\n- " names))
   end
 
-let is_installed config (distro:distribution) elem =
-  match Element.quick_test_file elem with
-  | None ->
-      let package_name = Element.package elem in
-      distro#is_valid_package_name package_name && distro#is_installed elem
-  | Some file ->
-      match config.system#stat file with
-      | None -> false
-      | Some info ->
-          match Element.quick_test_mtime elem with
-          | None -> true      (* quick-test-file exists and we don't care about the time *)
-          | Some required_mtime -> (Int64.of_float info.Unix.st_mtime) = required_mtime
-
 let install_distro_packages (distro:distribution) ui impls : [ `ok | `cancel ] Lwt.t =
   let groups = ref StringMap.empty in
   impls |> List.iter (fun impl ->
@@ -393,7 +349,7 @@ let install_distro_packages (distro:distribution) ui impls : [ `ok | `cancel ] L
   let rec loop = function
     | [] -> Lwt.return `ok
     | (typ, items) :: groups ->
-        match_lwt distro#install_distro_packages ui typ items with
+        distro#install_distro_packages ui typ items >>= function
         | `ok -> loop groups
         | `cancel -> Lwt.return `cancel in
   !groups |> StringMap.bindings |> loop
