@@ -11,6 +11,7 @@ type query = {
   elem_props : Impl.properties;     (* Properties on or inherited by the <package-element> - used by [add_package_implementation] *)
   feed : Feed.feed;                 (* The feed containing the <package-element> *)
   results : Impl.distro_implementation Support.Common.StringMap.t ref;
+  problem : string -> unit;
 }
 
 type quick_test_condition = Exists | UnchangedSince of float
@@ -35,32 +36,46 @@ class virtual distribution : General.config ->
     (** Paths to search for missing binaries (i.e. the platform default for $PATH) *)
     val system_paths : string list
 
-    val packagekit : Packagekit.packagekit
-
     (** Can we use packages for this distribution? For example, MacPortsDistribution can use "MacPorts" and "Darwin" packages. *)
     method match_name : string -> bool
 
+    (** Sometimes we don't know the correct path for a binary until the package is installed.
+     * Called by [add_package_implementation] when the package is already installed and may also be
+     * used by [install_distro_packages] when a package becomes installed.
+     * The default implementation checks for a "run" command and calls
+     * [get_correct_main] to get the correct value. *)
+    method private fixup_main : Impl.distro_implementation -> unit
+
     (** Test whether this <selection> element is still valid. The default implementation tries to load the feed from the
      * feed cache, calls [distribution#get_impls_for_feed] on it and checks whether the required implementation ID is in the
      * returned map. Override this if you can provide a more efficient implementation. *)
-    method is_installed : Selections.selection -> bool
+    method private is_installed : Selections.selection -> bool
+
+    (** Check whether this <selection> is still valid. If the quick-test-* attributes are present, we use
+        them to check. Otherwise, we call [is_installed]. *)
+    method is_installed_quick : Selections.selection -> bool
 
     (** Add the implementations for this feed to [query].
-     * Called by [get_impls_for_feed] once for each <package-implementation> element.
-     * This default implementation adds anything found previously by PackageKit. *)
-    method private get_package_impls : query -> unit
+     * Called by [get_impls_for_feed] once for each <package-implementation> element. *)
+    method virtual private get_package_impls : query -> unit
 
     (** Get the native implementations (installed or candidates for installation) for this feed.
      * This default implementation finds the best <package-implementation> elements and calls [get_package_impls] on each one.
-     * @param init add the results to this map, rather than starting with an empty one *)
-    method get_impls_for_feed : ?init:(Impl.distro_implementation Support.Common.StringMap.t) -> Feed.feed -> Impl.distro_implementation Support.Common.StringMap.t
+     * @param init add the results to this map, rather than starting with an empty one
+     * @param problem called to add warnings or notes about problems, for diagnostics *)
+    method get_impls_for_feed :
+      ?init:(Impl.distro_implementation Support.Common.StringMap.t) ->
+      problem:(string -> unit) ->
+      Feed.feed ->
+      Impl.distro_implementation Support.Common.StringMap.t
 
     (** Check (asynchronously) for available but currently uninstalled candidates. Once the returned
         promise resolves, the candidates should be included in future responses from [get_package_impls]. *)
-    method check_for_candidates : 'a. ui:(#Packagekit.ui as 'a) -> Feed.feed -> unit Lwt.t
+    method virtual check_for_candidates : 'a. ui:(#Packagekit.ui as 'a) -> Feed.feed -> unit Lwt.t
 
     (** Install a set of packages of a given type (as set previously by [check_for_candidates]).
-     * Normally called only by the [Distro.install_distro_packages] function. *)
+     * Normally called only by the [Distro.install_distro_packages] function.
+     * The default implementation tells the user to install them manually using [Impl.distro_retrieval_method]. *)
     method install_distro_packages : 'a. (#Packagekit.ui as 'a) -> string -> (Impl.distro_implementation * Impl.distro_retrieval_method) list -> [ `ok | `cancel ] Lwt.t
 
     (** Check whether this name is possible for this distribution. The default implementation filters using [valid_package_name]. *)
@@ -78,20 +93,16 @@ class virtual distribution : General.config ->
       ?main:string ->
       query ->
       version:Version.t ->
-      machine:(string option) ->
+      machine:(Arch.machine option) ->
       quick_test:(quick_test option) ->  (* The result is valid while this condition holds *)
       package_state:Impl.package_state ->
       distro_name:string ->
       unit
   end
 
-(** Check whether this <selection> is still valid. If the quick-test-* attributes are present, we use
-    them to check. Otherwise, we call [distribution#is_installed]. *)
-val is_installed : General.config -> distribution -> Selections.selection -> bool
-
 (** Install these packages using the distribution's package manager.
  * Sorts the implementations into groups by their type and then calls [distribution#install_distro_packages] once for each group. *)
 val install_distro_packages : distribution -> #Packagekit.ui -> Impl.distro_implementation list -> [ `ok | `cancel ] Lwt.t
 
 (** Return the <package-implementation> elements that best match this distribution. *)
-val get_matching_package_impls : distribution -> Feed.feed -> ([`package_impl] Element.t * Impl.properties) list
+val get_matching_package_impls : #distribution -> Feed.feed -> ([`package_impl] Element.t * Impl.properties) list
