@@ -11,9 +11,70 @@ module U = Support.Utils
 
 open Distro
 
+(** Base class for platforms that can use PackageKit *)
+class virtual packagekit_distro config =
+  object (self)
+    inherit distribution config
+
+    val packagekit = !Packagekit.packagekit (Support.Locale.LangMap.choose config.langs |> fst)
+
+    method private get_package_impls query =
+      let package_name = query.package_name in
+      let pk_unavailable reason = query.problem (Printf.sprintf "%s: %s" package_name reason) in
+      match Lwt.state packagekit#status with
+      | Lwt.Fail ex -> pk_unavailable (Printexc.to_string ex)
+      | Lwt.Sleep -> pk_unavailable "Waiting for PackageKit..."
+      | Lwt.Return (`Unavailable reason) -> pk_unavailable reason
+      | Lwt.Return `Ok ->
+          let pk_query = packagekit#get_impls package_name in
+          pk_query.Packagekit.problems |> List.iter query.problem;
+          pk_query.Packagekit.results |> List.iter (fun info ->
+            let {Packagekit.version; Packagekit.machine; Packagekit.installed; Packagekit.retrieval_method} = info in
+            let package_state =
+              if installed then `installed
+              else `uninstalled retrieval_method in
+            self#add_package_implementation
+              ~version
+              ~machine
+              ~package_state
+              ~quick_test:None
+              ~distro_name:distro_name
+              query
+          )
+
+    (* This default implementation queries PackageKit, if available. *)
+    method check_for_candidates ~ui feed =
+      match get_matching_package_impls self feed with
+      | [] -> Lwt.return ()
+      | matches ->
+          packagekit#status >>= function
+          | `Unavailable _ -> Lwt.return ()
+          | `Ok ->
+              let package_names = matches |> List.map (fun (elem, _props) -> Element.package elem) in
+              let hint = Feed_url.format_url feed.Feed.url in
+              packagekit#check_for_candidates ~ui ~hint package_names
+
+    method! install_distro_packages ui typ items =
+      match typ with
+      | "packagekit" ->
+          begin packagekit#install_packages ui items >>= function
+          | `cancel -> Lwt.return `cancel
+          | `ok ->
+              items |> List.iter (fun (impl, _rm) ->
+                self#fixup_main impl
+              );
+              Lwt.return `ok end
+      | _ ->
+          let names = items |> List.map (fun (_impl, rm) -> snd rm.Impl.distro_install_info) in
+          ui#confirm (Printf.sprintf
+            "This program depends on some packages that are available through your distribution. \
+             Please install them manually using %s before continuing. Or, install 'packagekit' and I can \
+             use that to install things. The packages are:\n\n- %s" typ (String.concat "\n- " names))
+  end
+
 let generic_distribution config =
   object
-    inherit Distro.distribution config
+    inherit packagekit_distro config
     val check_host_python = true
     val distro_name = "fallback"
     val id_prefix = "package:fallback"
@@ -33,14 +94,14 @@ let iter_dir system fn path =
 (** Lookup [elem]'s package in the cache. Generate the ID(s) for the cached implementations and check that one of them
     matches the [id] attribute on [elem].
     Returns [false] if the cache is out-of-date. *)
-let check_cache id_prefix elem (cache:Distro_cache.cache) =
+let check_cache id_prefix elem cache =
   let package = Element.package elem in
   let sel_id = Element.id elem in
   let matches (installed_version, machine) =
-    let installed_id = Printf.sprintf "%s:%s:%a:%s" id_prefix package Version.fmt installed_version (default "*" machine) in
+    let installed_id = Printf.sprintf "%s:%s:%a:%s" id_prefix package Version.fmt installed_version (Arch.format_machine_or_star machine) in
     (* log_warning "Want %s %s, have %s" package sel_id installed_id; *)
     sel_id = installed_id in
-  List.exists matches (fst (cache#get package))
+  List.exists matches (fst (Distro_cache.get cache package))
 
 module Debian = struct
   let dpkg_db_status = "/var/lib/dpkg/status"
@@ -116,7 +177,7 @@ module Debian = struct
                         match try_cleanup_distro_version_warn version package_name with
                         | None -> ()
                         | Some clean_version ->
-                            let r = (clean_version, (Arch.none_if_star (Support.System.canonical_machine (trim debarch)))) in
+                            let r = (clean_version, (Arch.parse_machine (Support.System.canonical_machine (trim debarch)))) in
                             results := r :: !results
                       )
                   | _ -> log_warning "Can't parse dpkg output: '%s'" line
@@ -127,56 +188,61 @@ module Debian = struct
       !results in
 
     let fixup_java_main impl java_version =
-      let java_arch = if impl.Impl.machine = Some "x86_64" then Some "amd64" else impl.Impl.machine in
+      let java_arch =
+        match Arch.format_machine_or_star impl.Impl.machine with
+        | "x86_64" -> Some "amd64"
+        | "*" -> log_warning "BUG: Missing machine type on Java!"; None
+        | m -> Some m in
 
-      match java_arch with
-      | None -> log_warning "BUG: Missing machine type on Java!"; None
-      | Some java_arch ->
-          let java_bin = Printf.sprintf "/usr/lib/jvm/java-%s-%s/jre/bin/java" java_version java_arch in
+      java_arch |> pipe_some (fun java_arch ->
+        let java_bin = Printf.sprintf "/usr/lib/jvm/java-%s-%s/jre/bin/java" java_version java_arch in
+        if system#file_exists java_bin then Some java_bin
+        else (
+          (* Try without the arch... *)
+          let java_bin = Printf.sprintf "/usr/lib/jvm/java-%s/jre/bin/java" java_version in
           if system#file_exists java_bin then Some java_bin
           else (
-            (* Try without the arch... *)
-            let java_bin = Printf.sprintf "/usr/lib/jvm/java-%s/jre/bin/java" java_version in
-            if system#file_exists java_bin then Some java_bin
-            else (
-              log_info "Java binary not found (%s)" java_bin;
-              Some "/usr/bin/java"
-            )
-          ) in
+            log_info "Java binary not found (%s)" java_bin;
+            Some "/usr/bin/java"
+          )
+        )
+      ) in
 
     object (self : #Distro.distribution)
-      inherit Distro.distribution config as super
+      inherit packagekit_distro config as super
       val check_host_python = false
 
       val distro_name = "Debian"
       val id_prefix = "package:deb"
-      val cache = new Distro_cache.cache config ~cache_leaf:"dpkg-status2.cache" status_file
+      val cache = Distro_cache.create_lazy config ~cache_leaf:"dpkg-status2.cache" ~source:status_file ~if_missing:query_dpkg
 
       (* If we added apt_cache results AND package kit is unavailable, this is set
        * so that we include them in the results. Otherwise, we just take the PackageKit results. *)
       val mutable use_apt_cache_results = false
 
-      method! is_installed elem =
+      method! private is_installed elem =
         check_cache id_prefix elem cache || super#is_installed elem
 
       method! private get_package_impls query =
-        (* Add any PackageKit candidates *)
-        super#get_package_impls query;
-
         let package_name = query.package_name in
-
-        (* Add apt-cache candidates (only if we're not using PackageKit) *)
-        if use_apt_cache_results then (
-          let entry = try Hashtbl.find apt_cache package_name with Not_found -> None in
-          entry |> if_some (fun {version; machine; size = _} ->
-            let machine = Arch.none_if_star machine in
-            let package_state = `uninstalled Impl.({distro_size = None; distro_install_info = ("apt-get install", package_name)}) in
-            self#add_package_implementation ~package_state ~version ~machine ~quick_test:None ~distro_name query
-          )
-        );
-
+        (* Add any PackageKit candidates *)
+        begin match Lwt.state packagekit#status with
+        | Lwt.Return `Ok | Lwt.Fail _ -> super#get_package_impls query;
+        | Lwt.Sleep -> ()   (* Only use apt-cache once we know PackageKit is missing *)
+        | Lwt.Return (`Unavailable _) ->
+            (* Add apt-cache candidates if we're not using PackageKit *)
+            try
+              match Hashtbl.find apt_cache package_name with
+              | Some {version; machine; size = _} ->
+                  let machine = Arch.parse_machine machine in
+                  let package_state = `uninstalled Impl.({distro_size = None; distro_install_info = ("apt-get install", package_name)}) in
+                  self#add_package_implementation ~package_state ~version ~machine ~quick_test:None ~distro_name query
+              | None ->
+                  query.problem (Printf.sprintf "%s: not known to apt-cache" package_name)
+            with Not_found -> ()  (* We haven't checked yet *)
+        end;
         (* Add installed packages by querying dpkg. *)
-        let infos, quick_test = cache#get ~if_missing:query_dpkg package_name in
+        let infos, quick_test = Distro_cache.get cache package_name in
         infos |> List.iter (fun (version, machine) ->
           self#add_package_implementation ~package_state:`installed ~version ~machine ~quick_test ~distro_name query
         )
@@ -191,18 +257,18 @@ module Debian = struct
              * results directly. If it is available, we'll use these results to filter the PackageKit query, because
              * it doesn't like queries for missing packages (it tends to abort the query early and miss some results). *)
             lwt () = query_apt_cache package_names
-            and pkgkit_available = packagekit#is_available in
+            and pkgkit_status = packagekit#status in
 
-            if pkgkit_available then (
-              let hint = Feed_url.format_url feed.Feed.url in
-              package_names
-              |> List.filter (Hashtbl.mem apt_cache)
-              |> packagekit#check_for_candidates ~ui ~hint
-            ) else (
-              (* No PackageKit. Use apt-cache directly. *)
-              use_apt_cache_results <- true;
-              Lwt.return ()
-            )
+            match pkgkit_status with
+            | `Ok ->
+                let hint = Feed_url.format_url feed.Feed.url in
+                package_names
+                |> List.filter (Hashtbl.mem apt_cache)
+                |> packagekit#check_for_candidates ~ui ~hint
+            | `Unavailable _ ->
+                (* No PackageKit. Use apt-cache directly. *)
+                use_apt_cache_results <- true;
+                Lwt.return ()
 
       method! private add_package_implementation ?id ?main query ~version ~machine ~quick_test ~package_state ~distro_name =
         let version =
@@ -236,7 +302,7 @@ module RPM = struct
       match impl.Impl.machine with
       | None -> log_warning "BUG: Missing machine type on Java!"; None
       | Some java_arch ->
-          let java_bin = Printf.sprintf "/usr/lib/jvm/jre-%s.%s/bin/java" java_version java_arch in
+          let java_bin = Printf.sprintf "/usr/lib/jvm/jre-%s.%s/bin/java" java_version (Arch.format_machine java_arch) in
           if config.system#file_exists java_bin then Some java_bin
           else (
             (* Try without the arch... *)
@@ -248,45 +314,43 @@ module RPM = struct
             )
           ) in
 
+    let regenerate add_entry =
+      ["rpm"; "-qa"; "--qf=%{NAME}\t%{VERSION}-%{RELEASE}\t%{ARCH}\n"]
+      |> U.check_output config.system (fun from_rpm  ->
+        try
+          while true do
+            let line = input_line from_rpm in
+            match Str.bounded_split_delim U.re_tab line 3 with
+            | ["gpg-pubkey"; _; _] -> ()
+            | [package; version; rpmarch] ->
+                let zi_arch = Support.System.canonical_machine (trim rpmarch) |> Arch.parse_machine in
+                try_cleanup_distro_version_warn version package |> if_some (fun clean_version ->
+                  add_entry package (clean_version, zi_arch)
+                )
+            | _ -> log_warning "Invalid output from 'rpm': %s" line
+          done
+        with End_of_file -> ()
+      ) in
+
     object (self)
-      inherit Distro.distribution config as super
+      inherit packagekit_distro config as super
       val check_host_python = false
 
       val distro_name = "RPM"
       val id_prefix = "package:rpm"
-      val cache =
-        object
-          inherit Distro_cache.cache config ~cache_leaf:"rpm-status3.cache" rpm_db_packages
-          method! private regenerate_cache add_entry =
-            ["rpm"; "-qa"; "--qf=%{NAME}\t%{VERSION}-%{RELEASE}\t%{ARCH}\n"]
-              |> U.check_output config.system (fun from_rpm  ->
-                try
-                  while true do
-                    let line = input_line from_rpm in
-                    match Str.bounded_split_delim U.re_tab line 3 with
-                    | ["gpg-pubkey"; _; _] -> ()
-                    | [package; version; rpmarch] ->
-                        let zi_arch = Support.System.canonical_machine (trim rpmarch) |> Arch.none_if_star in
-                        try_cleanup_distro_version_warn version package |> if_some (fun clean_version ->
-                          add_entry package (clean_version, zi_arch)
-                        )
-                    | _ -> log_warning "Invalid output from 'rpm': %s" line
-                  done
-                with End_of_file -> ()
-              )
-        end
+      val cache = Distro_cache.create_eager config ~cache_leaf:"rpm-status3.cache" ~source:rpm_db_packages ~regenerate
 
       method! private get_package_impls query =
         (* Add any PackageKit candidates *)
         super#get_package_impls query;
 
         (* Add installed packages by querying rpm *)
-        let infos, quick_test = cache#get query.package_name in
+        let infos, quick_test = Distro_cache.get cache query.package_name in
         infos |> List.iter (fun (version, machine) ->
           self#add_package_implementation ~package_state:`installed ~version ~machine ~quick_test ~distro_name query
         )
 
-      method! is_installed elem =
+      method! private is_installed elem =
         check_cache id_prefix elem cache || super#is_installed elem
 
       method! private get_correct_main impl run_command =
@@ -364,7 +428,7 @@ module ArchLinux = struct
       | _ -> items in
 
     object (self : #Distro.distribution)
-      inherit Distro.distribution config as super
+      inherit packagekit_distro config as super
       val check_host_python = false
 
       val distro_name = "Arch"
@@ -391,7 +455,7 @@ module ArchLinux = struct
                 match try_cleanup_distro_version_warn version package_name with
                 | None -> ()
                 | Some version ->
-                    let machine = Arch.none_if_star machine in
+                    let machine = Arch.parse_machine machine in
                     let quick_test = Some (desc_path, Exists) in
                     self#add_package_implementation ~package_state:`installed ~version ~machine ~quick_test ~distro_name query
     end
@@ -411,34 +475,42 @@ module Mac = struct
     let java_home version arch =
       U.finally_do Unix.close (Unix.openfile Support.System.dev_null [Unix.O_WRONLY] 0)
         (fun dev_null ->
-          let command = ["/usr/libexec/java_home"; "--failfast"; "--version"; version; "--arch"; arch] in
-          trim (try command |> U.check_output config.system ~stderr:(`FD dev_null) input_line with End_of_file -> "")
+          let reaper pid = Support.System.waitpid_non_intr pid |> ignore in
+          ["/usr/libexec/java_home"; "--failfast"; "--version"; version; "--arch"; arch]
+          |> U.check_output config.system ~reaper ~stderr:(`FD dev_null) (fun ch -> try input_line ch with End_of_file -> "")
+          |> trim
         ) in
 
     object (self : #distribution)
-      inherit Distro.distribution config as super
+      inherit Distro.distribution config
       val distro_name = "Darwin"
       val id_prefix = "package:darwin"
       val check_host_python = true
 
-      method! private get_package_impls query =
+      method get_package_impls query =
         match query.package_name with
         | "openjdk-6-jre" | "openjdk-6-jdk" -> self#find_java "1.6" "6" query
         | "openjdk-7-jre" | "openjdk-7-jdk" -> self#find_java "1.7" "7" query
+        | "openjdk-8-jre" | "openjdk-8-jdk" -> self#find_java "1.8" "8" query
         | "gnupg" -> self#find_program "/usr/local/bin/gpg" query
         | "gnupg2" -> self#find_program "/usr/local/bin/gpg2" query
-        | _ -> super#get_package_impls query
+        | "make" -> self#find_program "/usr/bin/make" query
+        | package_name ->
+            if StringMap.is_empty !(query.results) then
+              query.problem (Printf.sprintf "%s: Unknown Darwin/OS X package" package_name)
+            (* else we have MacPorts results *)
 
       method private find_program main query =
         config.system#stat main |> if_some (fun info ->
           let x_ok = try Unix.access main [Unix.X_OK]; true with Unix.Unix_error _ -> false in
           if x_ok then (
+            let (_host_os, host_machine) = Arch.platform config.system in
             try_cleanup_distro_version_warn (get_version main) query.package_name |> if_some (fun version ->
               self#add_package_implementation
                 ~main
                 ~package_state:`installed
                 ~version
-                ~machine:(Some config.system#platform.Platform.machine)
+                ~machine:(Some host_machine)
                 ~quick_test:(Some (main, UnchangedSince info.Unix.st_mtime))
                 ~distro_name
                 query
@@ -456,17 +528,55 @@ module Mac = struct
                 ~main
                 ~package_state:`installed
                 ~version:(Version.parse zero_version)
-                ~machine:(Some machine)
+                ~machine:(Arch.parse_machine machine)
                 ~quick_test:(Some (main, UnchangedSince info.Unix.st_mtime))
                 ~distro_name
                 query
           | None -> ()
         )
+
+      method check_for_candidates ~ui:_ _feed = Lwt.return ()
     end
 
   let macports_distribution ?(macports_db=macports_db) config =
     let re_version = Str.regexp "^@*\\([^+]+\\)\\(\\+.*\\)?$" in (* strip variants *)
     let re_extra = Str.regexp " platform='\\([^' ]*\\)\\( [0-9]+\\)?' archs='\\([^']*\\)'" in
+
+    let regenerate add_entry =
+      ["port"; "-v"; "installed"] |> U.check_output config.system (fun ch ->
+        try
+          while true do
+            let line = input_line ch in
+            log_debug "Got: '%s'" line;
+            if U.starts_with line " " then (
+              let line = trim line in
+              match Str.bounded_split_delim U.re_space line 3 with
+              | [package; version; extra] when U.starts_with extra "(active)" ->
+                  log_debug "Found package='%s' version='%s' extra='%s'" package version extra;
+                  if Str.string_match re_version version 0 then (
+                    let version = Str.matched_group 1 version in
+                    try_cleanup_distro_version_warn version package |> if_some (fun version ->
+                      if Str.string_match re_extra extra 0 then (
+                        (* let platform = Str.matched_group 1 extra in *)
+                        (* let major = Str.matched_group 2 extra in *)
+                        let archs = Str.matched_group 3 extra in
+                        Str.split U.re_space archs |> List.iter (fun arch ->
+                          let zi_arch = Support.System.canonical_machine arch in
+                          add_entry package (version, Arch.parse_machine zi_arch)
+                        )
+                      ) else (
+                        add_entry package (version, None)
+                      )
+                    )
+                  ) else log_debug "Failed to match version '%s'" version
+              | [_package; _version; _extra] -> ()
+              | [_package; _version] -> ()
+              | _ -> raise_safe "Invalid port output: '%s'" line
+            )
+          done
+        with End_of_file -> ()
+      ) in
+
     object (self : #distribution)
       inherit Distro.distribution config as super
 
@@ -476,64 +586,24 @@ module Mac = struct
 
       val distro_name = "MacPorts"
       val id_prefix = "package:macports"
-      val cache =
-        object
-          inherit Distro_cache.cache config ~cache_leaf:"macports-status2.cache" macports_db
+      val cache = Distro_cache.create_eager config ~cache_leaf:"macports-status2.cache" ~source:macports_db ~regenerate
 
-          method! private regenerate_cache add_entry =
-            ["port"; "-v"; "installed"] |> U.check_output config.system (fun ch ->
-              try
-                while true do
-                  let line = input_line ch in
-                  log_debug "Got: '%s'" line;
-                  if U.starts_with line " " then (
-                    let line = trim line in
-                    match Str.bounded_split_delim U.re_space line 3 with
-                    | [package; version; extra] when U.starts_with extra "(active)" ->
-                        log_debug "Found package='%s' version='%s' extra='%s'" package version extra;
-                        if Str.string_match re_version version 0 then (
-                          let version = Str.matched_group 1 version in
-                          try_cleanup_distro_version_warn version package |> if_some (fun version ->
-                            if Str.string_match re_extra extra 0 then (
-                              (* let platform = Str.matched_group 1 extra in *)
-                              (* let major = Str.matched_group 2 extra in *)
-                              let archs = Str.matched_group 3 extra in
-                              Str.split U.re_space archs |> List.iter (fun arch ->
-                                let zi_arch = Support.System.canonical_machine arch in
-                                add_entry package (version, Arch.none_if_star zi_arch)
-                              )
-                            ) else (
-                              add_entry package (version, None)
-                            )
-                          )
-                        ) else log_debug "Failed to match version '%s'" version
-                    | [_package; _version; _extra] -> ()
-                    | [_package; _version] -> ()
-                    | _ -> raise_safe "Invalid port output: '%s'" line
-                  )
-                done
-              with End_of_file -> ()
-            )
-        end
-
-      method! is_installed elem =
+      method! private is_installed elem =
         check_cache id_prefix elem cache || super#is_installed elem
 
       method! match_name name = super#match_name name || darwin#match_name name
 
-      method! get_impls_for_feed ?init feed =
-        let ports = super#get_impls_for_feed ?init feed in
-        darwin#get_impls_for_feed ~init:ports feed
-
-      method! private get_package_impls query =
-        super#get_package_impls query;
-
-        let infos, quick_test = cache#get query.package_name in
+      method private get_package_impls query =
+        let infos, quick_test = Distro_cache.get cache query.package_name in
         infos |> List.iter (fun (version, machine) ->
           self#add_package_implementation ~package_state:`installed ~version ~machine ~quick_test ~distro_name query
-        )
+        );
+        darwin#get_package_impls query
+
+      method check_for_candidates ~ui:_ _feed = Lwt.return ()
     end
 
+  let darwin_distribution config = (darwin_distribution config :> Distro.distribution)
 end
 
 module Win = struct
@@ -552,7 +622,7 @@ module Win = struct
           (value32, None) in
 
     object (self)
-      inherit Distro.distribution config as super
+      inherit Distro.distribution config
       val check_host_python = false (* (0install's bundled Python may not be generally usable) *)
 
       val! system_paths = []
@@ -560,9 +630,7 @@ module Win = struct
       val distro_name = "Windows"
       val id_prefix = "package:windows"
 
-      method! private get_package_impls query =
-        super#get_package_impls query;
-
+      method private get_package_impls query =
         let package_name = query.package_name in
         match package_name with
         | "openjdk-6-jre" -> self#find_java "Java Runtime Environment" "1.6" "6" query
@@ -579,10 +647,10 @@ module Win = struct
         | "netfx-client" ->
             self#find_netfx "v4\\Client" "4.0" query;
             self#find_netfx_release "v4\\Client" 378389 "4.5" query;
-        | _ -> ()
+        | _ ->
+            query.problem (Printf.sprintf "%s: Unknown Windows package" package_name)
 
-        (* No PackageKit support on Windows *)
-      method! check_for_candidates ~ui:_ _feed = Lwt.return ()
+      method check_for_candidates ~ui:_ _feed = Lwt.return ()
 
       method private find_netfx win_version zero_version query =
         let reg_path = "SOFTWARE\\Microsoft\\NET Framework Setup\\NDP\\" ^ win_version in
@@ -599,7 +667,7 @@ module Win = struct
                 ~main:""      (* .NET executables do not need a runner on Windows but they need one elsewhere *)
                 ~package_state
                 ~version
-                ~machine:(Some machine)
+                ~machine:(Arch.parse_machine machine)
                 ~quick_test:None
                 ~distro_name
                 query
@@ -620,7 +688,7 @@ module Win = struct
                 ~main:""      (* .NET executables do not need a runner on Windows but they need one elsewhere *)
                 ~package_state
                 ~version
-                ~machine:(Some machine)
+                ~machine:(Arch.parse_machine machine)
                 ~quick_test:None
                 ~distro_name
                 query
@@ -644,7 +712,7 @@ module Win = struct
                     ~main:java_bin
                     ~package_state:`installed
                     ~version
-                    ~machine:(Some machine)
+                    ~machine:(Arch.parse_machine machine)
                     ~quick_test
                     ~distro_name
                     query
@@ -656,41 +724,42 @@ module Win = struct
   (* Note: this is ported from the Python but completely untested. *)
   let cygwin_distribution config =
     let re_whitespace = Str.regexp "[ \t]+" in
+    let regenerate add_entry =
+      ["cygcheck"; "-c"; "-d"] |> U.check_output config.system (fun from_cyg  ->
+        try
+          while true do
+            match input_line from_cyg with
+            | "Cygwin Package Information" | "" -> ()
+            | line ->
+                match U.split_pair re_whitespace line with
+                | ("Package", "Version") -> ()
+                | (package, version) ->
+                    try_cleanup_distro_version_warn version package |> if_some (fun clean_version ->
+                      add_entry package (clean_version, None)
+                    )
+          done
+        with End_of_file -> ()
+      ) in
     object (self)
       inherit Distro.distribution config as super
       val distro_name = "Cygwin"
       val id_prefix = "package:cygwin"
       val check_host_python = false
 
-      val cache =
-        object
-          inherit Distro_cache.cache config ~cache_leaf:"cygcheck-status2.cache" cygwin_log
-          method! private regenerate_cache add_entry =
-            ["cygcheck"; "-c"; "-d"] |> U.check_output config.system (fun from_cyg  ->
-                try
-                  while true do
-                    match input_line from_cyg with
-                    | "Cygwin Package Information" | "" -> ()
-                    | line ->
-                        match U.split_pair re_whitespace line with
-                        | ("Package", "Version") -> ()
-                        | (package, version) ->
-                            try_cleanup_distro_version_warn version package |> if_some (fun clean_version ->
-                              add_entry package (clean_version, None)
-                            )
-                  done
-                with End_of_file -> ()
-              )
-        end
+      val cache = Distro_cache.create_eager config ~cache_leaf:"cygcheck-status2.cache" ~source:cygwin_log ~regenerate
 
-      method! is_installed elem =
+      method! private is_installed elem =
         check_cache id_prefix elem cache || super#is_installed elem
 
-      method! private get_package_impls query =
-        let infos, quick_test = cache#get query.package_name in
+      method private get_package_impls query =
+        match Distro_cache.get cache query.package_name with
+        | [], _ -> query.problem (Printf.sprintf "%s: Unknown Cygwin package" query.package_name)
+        | infos, quick_test ->
         infos |> List.iter (fun (version, machine) ->
           self#add_package_implementation ~package_state:`installed ~version ~machine ~quick_test ~distro_name query
         )
+
+      method check_for_candidates ~ui:_ _feed = Lwt.return ()
     end
 end
 
@@ -701,7 +770,7 @@ module Ports = struct
     let re_name_version = Str.regexp "^\\(.+\\)-\\([^-]+\\)$" in
 
     object (self)
-      inherit Distro.distribution config
+      inherit packagekit_distro config  (* Can ports use PackageKit? Not sure. *)
       val id_prefix = "package:ports"
       val distro_name = "Ports"
       val check_host_python = true
@@ -715,18 +784,18 @@ module Ports = struct
               let version = Str.matched_group 2 pkgname in
               if name = query.package_name then (
                 try_cleanup_distro_version_warn version query.package_name |> if_some (fun version ->
-                  let machine = Some config.system#platform.Platform.machine in
+                  let (_host_os, host_machine) = Arch.platform config.system in
                   self#add_package_implementation
                     ~package_state:`installed
                     ~version
-                    ~machine
+                    ~machine:(Some host_machine)
                     ~quick_test:None
                     ~distro_name
                     query
                 )
               )
             ) else (
-              log_warning "Cannot parse version from Ports package named '%s'" pkgname
+              query.problem (Printf.sprintf "Cannot parse version from Ports package named '%s'" pkgname)
             )
           )
         )
@@ -740,13 +809,16 @@ module Gentoo = struct
 
   let gentoo_distribution ?(pkgdir=Ports.pkg_db) config =
     object (self)
-      inherit Distro.distribution config
+      inherit packagekit_distro config as super
       val! valid_package_name = Str.regexp "^[^.-][^/]*/[^./][^/]*$"
       val distro_name = "Gentoo"
       val id_prefix = "package:gentoo"
       val check_host_python = false
 
       method! private get_package_impls query =
+        (* Add any PackageKit candidates *)
+        super#get_package_impls query;
+
         let re_version_start = Str.regexp "-[0-9]" in
 
         match Str.bounded_split_delim U.re_slash query.package_name 2 with
@@ -774,7 +846,7 @@ module Gentoo = struct
                           input_line ch |> U.split_pair U.re_dash |> fst
                         )
                       ) in
-                    let machine = Arch.none_if_star (Support.System.canonical_machine machine) in
+                    let machine = Arch.parse_machine (Support.System.canonical_machine machine) in
                     self#add_package_implementation
                       ~package_state:`installed
                       ~version
@@ -794,16 +866,18 @@ module Slackware = struct
 
   let slack_distribution ?(packages_dir=slack_db) config =
     object (self)
-      inherit Distro.distribution config
+      inherit packagekit_distro config as super
       val distro_name = "Slack"
       val id_prefix = "package:slack"
       val check_host_python = false
 
       method! private get_package_impls query =
+        (* Add any PackageKit candidates *)
+        super#get_package_impls query;
         packages_dir |> iter_dir config.system (fun entry ->
           match Str.bounded_split_delim U.re_dash entry 4 with
           | [name; version; arch; build] when name = query.package_name ->
-              let machine = Arch.none_if_star (Support.System.canonical_machine arch) in
+              let machine = Arch.parse_machine (Support.System.canonical_machine arch) in
               try_cleanup_distro_version_warn (version ^ "-" ^ build) query.package_name |> if_some (fun version ->
               self#add_package_implementation
                 ~package_state:`installed
@@ -843,10 +917,12 @@ let get_host_distribution config : Distro.distribution =
           Ports.ports_distribution config
       ) else if exists Slackware.slack_db then
         Slackware.slack_distribution config
-      else if config.system#platform.Platform.os = "Darwin" then
-        Mac.darwin_distribution config
-      else
-        generic_distribution config
+      else begin match config.system#platform.Platform.os with
+      | "Darwin" | "MacOSX" ->
+          Mac.darwin_distribution config
+      | _unknown ->
+          generic_distribution config
+      end
   | "Win32" -> Win.windows_distribution config
   | "Cygwin" -> Win.cygwin_distribution config
   | _ ->
