@@ -26,9 +26,9 @@ type feed_type =
 type feed_import = {
   feed_src : Feed_url.non_distro_feed;
 
-  feed_os : string option;          (* All impls requires this OS *)
-  feed_machine : string option;     (* All impls requires this CPU *)
-  feed_langs : string list option;  (* No impls for languages not listed *)
+  feed_os : Arch.os option;           (* All impls requires this OS *)
+  feed_machine : Arch.machine option; (* All impls requires this CPU *)
+  feed_langs : string list option;    (* No impls for languages not listed *)
   feed_type: feed_type;
 }
 
@@ -46,139 +46,146 @@ type feed = {
   package_implementations : ([`package_impl] Element.t * Impl.properties) list;
 }
 
-let parse_implementations (system:system) url root local_dir =
+let create_impl system ~local_dir state node =
+  let open Impl in
+  let s = ref state in
+
+  let set_attr name value =
+    let new_attrs = AttrMap.add_no_ns name value !s.Impl.attrs in
+    s := {!s with Impl.attrs = new_attrs} in
+
+  let get_required_attr name =
+    AttrMap.get_no_ns name !s.attrs |? lazy (Element.raise_elem "Missing attribute '%s' on" name node) in
+
+  let id = Element.id node in
+
+  let () =
+    match local_dir with
+    | None ->
+        if AttrMap.mem ("", FeedAttr.local_path) !s.attrs then
+          Element.raise_elem "local-path in non-local feed! " node;
+    | Some dir ->
+        let use rel_path =
+          if Filename.is_relative rel_path then
+            set_attr FeedAttr.local_path @@ Support.Utils.abspath system @@ dir +/ rel_path
+          else
+            set_attr FeedAttr.local_path rel_path in
+        match AttrMap.get_no_ns FeedAttr.local_path !s.attrs with
+        | Some path -> use path
+        | None ->
+            if Support.Utils.starts_with id "/" || Support.Utils.starts_with id "." then
+              use id in
+
+  (* version-modifier *)
+  AttrMap.get_no_ns FeedAttr.version_modifier !s.attrs
+  |> if_some (fun modifier ->
+      let real_version = get_required_attr FeedAttr.version ^ modifier in
+      let new_attrs = AttrMap.add_no_ns FeedAttr.version real_version (AttrMap.remove ("", FeedAttr.version_modifier) !s.attrs) in
+      s := {!s with attrs = new_attrs}
+  );
+
+  let get_prop key =
+    AttrMap.get_no_ns key !s.attrs |? lazy (Element.raise_elem "Missing attribute '%s' on" key node) in
+
+  let (os, machine) =
+    try Arch.parse_arch @@ default "*-*" @@ AttrMap.get_no_ns "arch" !s.attrs
+    with Safe_exception _ as ex -> reraise_with_context ex "... processing %s" (Element.show_with_loc node) in
+
+  let stability =
+    match AttrMap.get_no_ns FeedAttr.stability !s.attrs with
+    | None -> Testing
+    | Some s -> parse_stability ~from_user:false s in
+
+  let impl_type =
+    match AttrMap.get_no_ns FeedAttr.local_path !s.attrs with
+    | Some local_path ->
+        assert (local_dir <> None);
+        `local_impl local_path
+    | None ->
+        let retrieval_methods = Element.retrieval_methods node in
+        `cache_impl { digests = Stores.get_digests node; retrieval_methods; } in
+
+  let impl = {
+    qdom = (node :> [ `implementation | `package_impl ] Element.t);
+    props = {!s with requires = List.rev !s.requires};
+    os;
+    machine;
+    stability;
+    parsed_version = Version.parse (get_prop FeedAttr.version);
+    impl_type;
+  } in
+  (id, impl)
+
+let process_group_properties ~local_dir state item =
+  let open Impl in
+  let s = ref state in
+  (* We've found a group or implementation. Scan for dependencies,
+      bindings and commands. Doing this here means that:
+      - We can share the code for groups and implementations here.
+      - The order doesn't matter, because these get processed first.
+      A side-effect is that the document root cannot contain these. *)
+
+  (* Upgrade main='...' to <command name='run' path='...'> etc *)
+  let add_command ?path ?shell_command name =
+    let new_command =
+      Element.make_command ~source_hint:(Some item) ?path ?shell_command name
+      |> Impl.parse_command local_dir in
+    s := {!s with commands = StringMap.add name new_command !s.commands} in
+
+  Element.main item |> if_some (fun path -> add_command ~path "run");
+  Element.self_test item |> if_some (fun path -> add_command ~path "test");
+  Element.compile_command item |> if_some (fun shell_command -> add_command ~shell_command "compile");
+
+  let new_bindings = ref [] in
+
+  Element.deps_and_bindings item |> List.iter (function
+    | `requires child  | `restricts child ->
+        let req = Impl.parse_dep local_dir child in
+        s := {!s with requires = req :: !s.requires}
+    | `command child ->
+        let command_name = Element.command_name child in
+        s := {!s with commands = StringMap.add command_name (Impl.parse_command local_dir child) !s.commands}
+    | #Element.binding as child ->
+        new_bindings := Element.element_of_binding child :: !new_bindings
+    | _ -> ()
+  );
+
+  if !new_bindings <> [] then
+    s := {!s with bindings = !s.bindings @ (List.rev !new_bindings)};
+
+  let new_attrs = !s.attrs |> AttrMap.add_all (Element.as_xml item).Q.attrs in
+
+  {!s with
+    attrs = new_attrs;
+    requires = !s.requires;
+  }
+
+let default_attrs ~url =
+  AttrMap.empty
+    |> AttrMap.add_no_ns FeedAttr.stability FeedAttr.value_testing
+    |> AttrMap.add_no_ns FeedAttr.from_feed url
+
+let parse_implementations (system:system) root_attrs root local_dir =
   let open Impl in
   let implementations = ref StringMap.empty in
   let package_implementations = ref [] in
 
   let process_impl node (state:Impl.properties) =
-    let s = ref state in
-
-    let set_attr name value =
-      let new_attrs = AttrMap.add_no_ns name value !s.Impl.attrs in
-      s := {!s with Impl.attrs = new_attrs} in
-
-    let get_required_attr name =
-      AttrMap.get_no_ns name !s.attrs |? lazy (Element.raise_elem "Missing attribute '%s' on" name node) in
-
-    let id = Element.id node in
-
-    let () =
-      match local_dir with
-      | None ->
-          if AttrMap.mem ("", FeedAttr.local_path) !s.attrs then
-            Element.raise_elem "local-path in non-local feed! " node;
-      | Some dir ->
-          let use rel_path =
-            if Filename.is_relative rel_path then
-              set_attr FeedAttr.local_path @@ Support.Utils.abspath system @@ dir +/ rel_path
-            else
-              set_attr FeedAttr.local_path rel_path in
-          match AttrMap.get_no_ns FeedAttr.local_path !s.attrs with
-          | Some path -> use path
-          | None ->
-              if Support.Utils.starts_with id "/" || Support.Utils.starts_with id "." then
-                use id in
-
+    let (id, impl) = create_impl system ~local_dir state node in
     if StringMap.mem id !implementations then
       Element.raise_elem "Duplicate ID '%s' in:" id node;
-    (* version-modifier *)
-    AttrMap.get_no_ns FeedAttr.version_modifier !s.attrs
-    |> if_some (fun modifier ->
-        let real_version = get_required_attr FeedAttr.version ^ modifier in
-        let new_attrs = AttrMap.add_no_ns FeedAttr.version real_version (AttrMap.remove ("", FeedAttr.version_modifier) !s.attrs) in
-        s := {!s with attrs = new_attrs}
-    );
-
-    let get_prop key =
-      AttrMap.get_no_ns key !s.attrs |? lazy (Element.raise_elem "Missing attribute '%s' on" key node) in
-
-    let (os, machine) =
-      try Arch.parse_arch @@ default "*-*" @@ AttrMap.get_no_ns "arch" !s.attrs
-      with Safe_exception _ as ex -> reraise_with_context ex "... processing %s" (Element.show_with_loc node) in
-
-    let stability =
-      match AttrMap.get_no_ns FeedAttr.stability !s.attrs with
-      | None -> Testing
-      | Some s -> parse_stability ~from_user:false s in
-
-    let impl_type =
-      match AttrMap.get_no_ns FeedAttr.local_path !s.attrs with
-      | Some local_path ->
-          assert (local_dir <> None);
-          `local_impl local_path
-      | None ->
-          let retrieval_methods = Element.retrieval_methods node in
-          `cache_impl { digests = Stores.get_digests node; retrieval_methods; } in
-
-    let impl = {
-      qdom = Element.as_xml node;
-      props = {!s with requires = List.rev !s.requires};
-      os;
-      machine;
-      stability;
-      parsed_version = Version.parse (get_prop FeedAttr.version);
-      impl_type;
-    } in
     implementations := StringMap.add id impl !implementations
   in
 
-  let process_item state item =
-    let s = ref state in
-    (* We've found a group or implementation. Scan for dependencies,
-       bindings and commands. Doing this here means that:
-       - We can share the code for groups and implementations here.
-       - The order doesn't matter, because these get processed first.
-       A side-effect is that the document root cannot contain these. *)
-
-    (* Upgrade main='...' to <command name='run' path='...'> etc *)
-    let add_command ?path ?shell_command name =
-      let new_command =
-        Element.make_command ~source_hint:(Some item) ?path ?shell_command name
-        |> Impl.parse_command local_dir in
-      s := {!s with commands = StringMap.add name new_command !s.commands} in
-
-    Element.main item |> if_some (fun path -> add_command ~path "run");
-    Element.self_test item |> if_some (fun path -> add_command ~path "test");
-    Element.compile_command item |> if_some (fun shell_command -> add_command ~shell_command "compile");
-
-    let new_bindings = ref [] in
-
-    Element.deps_and_bindings item |> List.iter (function
-      | `requires child  | `restricts child ->
-          let req = Impl.parse_dep local_dir child in
-          s := {!s with requires = req :: !s.requires}
-      | `command child ->
-          let command_name = Element.command_name child in
-          s := {!s with commands = StringMap.add command_name (Impl.parse_command local_dir child) !s.commands}
-      | #Element.binding as child ->
-          new_bindings := Element.element_of_binding child :: !new_bindings
-      | _ -> ()
-    );
-
-    if !new_bindings <> [] then
-      s := {!s with bindings = !s.bindings @ (List.rev !new_bindings)};
-
-    let new_attrs = !s.attrs |> AttrMap.add_all (Element.as_xml item).Q.attrs in
-
-    {!s with
-      attrs = new_attrs;
-      requires = !s.requires;
-    } in
-
   let rec process_group state group =
     Element.group_children group |> List.iter (function
-      | `group item -> process_group (process_item state item) (item :> [`feed | `group] Element.t)
-      | `implementation item -> process_impl item (process_item state item)
+      | `group item -> process_group (process_group_properties ~local_dir state item) (item :> [`feed | `group] Element.t)
+      | `implementation item -> process_impl item (process_group_properties ~local_dir state item)
       | `package_impl item ->
-          package_implementations := (item, (process_item state item)) :: !package_implementations
+          package_implementations := (item, (process_group_properties ~local_dir state item)) :: !package_implementations
     )
   in
 
-  let root_attrs = AttrMap.empty
-    |> AttrMap.add_no_ns FeedAttr.stability FeedAttr.value_testing
-    |> AttrMap.add_no_ns FeedAttr.from_feed url in
-
   (* 'main' on the <interface> (deprecated) *)
   let root_commands = match Element.main root with
     | None -> StringMap.empty
@@ -251,7 +258,8 @@ let parse system root feed_local_path =
     | `feed_for _ | `category _ | `needs_terminal _ | `icon _ | `homepage _ -> ()
   );
 
-  let implementations, package_implementations = parse_implementations system url root local_dir in
+  let implementations, package_implementations =
+    parse_implementations system (default_attrs ~url) root local_dir in
 
   {
     url = Feed_url.parse_non_distro url;
