@@ -22,9 +22,9 @@ type feed_type =
 type feed_import = {
   feed_src : Feed_url.non_distro_feed;
 
-  feed_os : string option;          (* All impls requires this OS *)
-  feed_machine : string option;     (* All impls requires this CPU *)
-  feed_langs : string list option;  (* No impls for languages not listed *)
+  feed_os : Arch.os option;           (* All impls requires this OS *)
+  feed_machine : Arch.machine option; (* All impls requires this CPU *)
+  feed_langs : string list option;    (* No impls for languages not listed *)
   feed_type : feed_type;
 }
 
@@ -45,8 +45,10 @@ type feed = {
 (** {2 Parsing} *)
 val parse : system -> [`feed] Element.t -> filepath option -> feed
 
-val get_implementations : feed -> Impl.generic_implementation list
-
+val get_implementations : feed -> Impl.existing Impl.t list
+val default_attrs : url:string -> Support.Qdom.AttrMap.t
+val process_group_properties : local_dir:filepath option -> Impl.properties ->
+  [<`group | `implementation | `package_impl] Element.t -> Impl.properties
 val load_feed_overrides : General.config -> [< Feed_url.parsed_feed_url] -> feed_overrides
 val save_feed_overrides : General.config -> [< Feed_url.parsed_feed_url] -> feed_overrides -> unit
 val update_last_checked_time : General.config -> [< Feed_url.remote_feed] -> unit
