@@ -4,11 +4,19 @@
 
 (** Provides feeds to the solver. *)
 
+open Support.Common
+
+type distro_impls = {
+  impls : Impl.distro_implementation StringMap.t;
+  overrides : Feed.feed_overrides;
+  problems : string list;
+}
+
 class type feed_provider =
   object
     method forget_distro : Feed_url.non_distro_feed -> unit
     method forget_user_feeds : General.iface_uri -> unit
-    method get_distro_impls : Feed.feed -> Impl.distro_implementation Support.Common.StringMap.t * Feed.feed_overrides
+    method get_distro_impls : Feed.feed -> distro_impls
     method get_feed : Feed_url.non_distro_feed -> (Feed.feed * Feed.feed_overrides) option
     method get_feeds_used : Feed_url.non_distro_feed list
     method get_iface_config : General.iface_uri -> Feed_cache.interface_config
