@@ -4,8 +4,19 @@
 
 (** Manage the GUI sub-process. *)
 
+type feed_description = {
+  times : (string * float) list;
+  summary : string option;
+  description : string list;
+  homepages : string list;
+  signatures : [
+    | `Valid of Support.Gpg.fingerprint * Support.Gpg.timestamp * string option * [`Trusted | `Not_trusted]
+    | `Invalid of string
+  ] list;
+}
+
 (** The GUI plugin registers itself here. *)
-val register_plugin : (General.config -> Support.Common.yes_no_maybe -> Ui.ui_handler option) -> unit
+val register_plugin : (General.config -> Ui.ui_handler option) -> unit
 
 val download_icon : Fetch.fetcher -> Feed_provider.feed_provider -> Feed_url.non_distro_feed -> unit Lwt.t
 
@@ -41,7 +52,7 @@ val have_source_for : Feed_provider.feed_provider -> General.iface_uri -> bool
 (** List the implementations of this interface in the order they should be shown in the GUI.
  * @return (selected_version, implementations). *)
 val list_impls : Solver.Model.t -> Solver.role ->
-  (Impl.generic_implementation option * (Impl.generic_implementation * Impl_provider.rejection option) list)
+  (Impl.generic_implementation option * (Impl.generic_implementation * Impl_provider.rejection_reason option) list)
 
 (* Returns (fetch-size, fetch-tooltip) *)
 val get_fetch_info : General.config -> Impl.generic_implementation -> (string * string)
@@ -58,3 +69,5 @@ val get_bug_report_details : General.config -> role:Solver.role -> (bool * Solve
 val send_bug_report : General.iface_uri -> string -> string Lwt.t
 
 val run_test : General.config -> Distro.distribution -> (Selections.t -> string Lwt.t) -> (bool * Solver.Model.t) -> string Lwt.t
+
+val generate_feed_description : General.config -> Trust.trust_db -> Feed.feed -> Feed.feed_overrides -> feed_description Lwt.t
