@@ -2,7 +2,6 @@
  * See the README file for details, or visit http://0install.net.
  *)
 
-open General
 open Support.Common
 module U = Support.Utils
 module IPackageKit = Packagekit_interfaces.Org_freedesktop_PackageKit
@@ -10,7 +9,7 @@ module ITrans = Packagekit_interfaces.Org_freedesktop_PackageKit_Transaction
 
 type package_info = {
   version : Version.t;
-  machine : string option;
+  machine : Arch.machine option;
   installed : bool;
   retrieval_method : Impl.distro_retrieval_method;
 }
@@ -18,6 +17,11 @@ type package_info = {
 type packagekit_id = string
 type size = Int64.t
 
+type query_result = {
+  results : package_info list;
+  problems : string list;
+}
+
 class type ui =
   object
     method monitor : Downloader.download -> unit
@@ -27,8 +31,8 @@ class type ui =
 
 class type packagekit =
   object
-    method is_available : bool Lwt.t
-    method get_impls : string -> package_info list
+    method status : [`Ok | `Unavailable of string] Lwt.t
+    method get_impls : string -> query_result
     method check_for_candidates : 'a. ui:(#ui as 'a) -> hint:string -> string list -> unit Lwt.t
     method install_packages : 'a. (#ui as 'a) -> (Impl.distro_implementation * Impl.distro_retrieval_method) list -> [ `ok | `cancel ] Lwt.t
   end
@@ -38,8 +42,7 @@ let max_batch_size = 100
 
 (** Wait for a task to complete, but ignore the actual return value. *)
 let lwt_wait_for task =
-  lwt _ = task in
-  Lwt.return ()
+  task >|= ignore
 
 (** [keep ~switch value] prevents [value] from being garbage collected until the switch is turned off.
  * See http://stackoverflow.com/questions/19975140/how-to-stop-ocaml-garbage-collecting-my-reactive-event-handler *)
@@ -58,9 +61,8 @@ let keep =
 (* We want to make lists of signal connection requests, so they all need the same type (and we can't used GADTs yet),
  * so we use closures rather than tuples. *)
 let make_signal_request signal handler switch proxy =
-  lwt event = Dbus.OBus_signal.(connect ~switch (make signal proxy)) in
-  Lwt_react.E.map (handler : _ -> unit) event |> keep ~switch;
-  Lwt.return ()
+  Dbus.OBus_signal.(connect ~switch (make signal proxy)) >|= fun event ->
+  Lwt_react.E.map (handler : _ -> unit) event |> keep ~switch
 
 (** Convert short names to PackageKit IDs (e.g. "gnupg" -> "gnupg;2.0.22;x86_64;arch") *)
 let resolve pk package_names =
@@ -70,21 +72,24 @@ let resolve pk package_names =
     log_info "packagekit: resolved %s: %s" package_id summary;
     match Str.bounded_split_delim U.re_semicolon package_id 4 with
     | [package_name; version; machine; repo] ->
-        begin match Version.try_cleanup_distro_version version with
-        | None -> log_warning "packagekit: failed to parse version string '%s' for '%s'" version package_name
-        | Some version ->
-            let machine = Support.System.canonical_machine machine in
-            let info = {
-              version;
-              machine = (if machine = "*" then None else Some machine);
-              installed = (repo = "installed");
-              retrieval_method = { Impl.
-                distro_size = None;    (* Gets filled in later *)
-                distro_install_info = ("packagekit", package_id)
+        Hashtbl.add details package_name begin
+          match Version.try_cleanup_distro_version version with
+          | None ->
+              `Error (Printf.sprintf "Failed to parse version string '%s' for '%s'" version package_name)
+          | Some version ->
+              let machine = Support.System.canonical_machine machine in
+              `Ok {
+                version;
+                machine = Arch.parse_machine machine;
+                installed = (repo = "installed");
+                retrieval_method = { Impl.
+                  distro_size = None;    (* Gets filled in later *)
+                  distro_install_info = ("packagekit", package_id)
+                }
               }
-            } in
-            Hashtbl.add details package_name info end
-    | _ -> log_warning "package_id: malformed package ID returned by PackageKit: '%s'" package_id in
+        end
+    | _ ->
+        log_warning "Malformed package ID returned by PackageKit: '%s'" package_id in
 
   lwt () =
     try_lwt
@@ -100,7 +105,7 @@ let resolve pk package_names =
     with Safe_exception _ as ex ->
       (* This is a bit broken. PackageKit seems to abort on the first unknown package, so we
        * lose the remaining results. Still, something is better than nothing... *)
-      log_debug ~ex "Error resolving names with PackageKit";
+      log_debug ~ex "Error resolving %s with PackageKit" (String.concat "," package_names);
       Lwt.return () in
 
   let add map name = map |> StringMap.add name (Hashtbl.find_all details name) in
@@ -120,10 +125,10 @@ let get_sizes pk = function
       let details_signal =
         if pk#version >= [0; 8; 1] then make_signal_request ITrans.s_Details2 update
         else make_signal_request ITrans.s_Details1 update in
-      lwt () = pk#run_transaction [details_signal] (fun _switch proxy ->
+      pk#run_transaction [details_signal] (fun _switch proxy ->
         Dbus.OBus_method.call ITrans.m_GetDetails proxy package_ids
-      ) in
-      Lwt.return !details
+      ) >|= fun () ->
+      !details
 
 let get_packagekit_id = function
   | {Impl.distro_install_info = ("packagekit", id); _} -> id
@@ -141,8 +146,8 @@ let install (ui:#ui) pk items =
   let finished, set_finished = Lwt_react.S.create false in
   try_lwt
     let cancelled = ref false in
-    lwt () = pk#run_transaction [] (fun switch proxy ->
-      lwt percentage = Dbus.OBus_property.monitor ~switch (Dbus.OBus_property.make ITrans.p_Percentage proxy) in
+    pk#run_transaction [] (fun switch proxy ->
+      Dbus.OBus_property.monitor ~switch (Dbus.OBus_property.make ITrans.p_Percentage proxy) >>= fun percentage ->
       let progress = Lwt_react.S.l2 (fun perc finished ->
         match total_size with
         | Some size when size <> Int64.zero ->
@@ -174,7 +179,7 @@ let install (ui:#ui) pk items =
         Dbus.OBus_method.call ITrans.m_InstallPackages2 proxy (Int64.zero, packagekit_ids)
       else
         Dbus.OBus_method.call ITrans.m_InstallPackages proxy (false, packagekit_ids)
-    ) in
+    ) >>= fun () ->
     (* Mark each package as now installed (possibly we should do this individually in a signal callback instead). *)
     items |> List.iter (fun (impl, _rm) ->
       let `package_impl info = impl.Impl.impl_type in
@@ -187,7 +192,7 @@ let install (ui:#ui) pk items =
     Lwt.return ()
 
 (** The top-level PackageKit service. *)
-let packagekit_service config proxy version =
+let packagekit_service lang_spec proxy version =
   object
     method version = version
 
@@ -196,26 +201,23 @@ let packagekit_service config proxy version =
         the switch. *)
     method run_transaction signals cb =
       (* Create transaction *)
-      lwt path =
-        if version >= [0;8;1] then
-          Dbus.OBus_method.call IPackageKit.m_CreateTransaction proxy ()
-        else (
-          lwt path_str = Dbus.OBus_method.call IPackageKit.m_GetTid proxy () in
-          Dbus.OBus_path.of_string path_str |> Lwt.return
-        ) in
+      begin if version >= [0;8;1] then
+        Dbus.OBus_method.call IPackageKit.m_CreateTransaction proxy ()
+      else
+        Dbus.OBus_method.call IPackageKit.m_GetTid proxy () >|= Dbus.OBus_path.of_string
+      end >>= fun path ->
       let peer = proxy.Dbus.OBus_proxy.peer in
       let trans_proxy = Dbus.OBus_proxy.make ~peer ~path in
 
       (* Set locale *)
-      let locale = Support.Locale.LangMap.choose config.langs |> fst |> Support.Locale.format_lang in
-      lwt () =
-        if version >= [0; 6; 0] then
-          Dbus.OBus_method.call ITrans.m_SetHints trans_proxy ["locale=" ^ locale]
-        else
-          Dbus.OBus_method.call ITrans.m_SetLocale trans_proxy locale in
+      let locale = Support.Locale.format_lang lang_spec in
+      begin if version >= [0; 6; 0] then
+        Dbus.OBus_method.call ITrans.m_SetHints trans_proxy ["locale=" ^ locale]
+      else
+        Dbus.OBus_method.call ITrans.m_SetLocale trans_proxy locale
+      end >>= fun () ->
 
-      let switch = Lwt_switch.create () in
-      try_lwt
+      U.with_switch (fun switch ->
         let error = ref None in
         let finished, waker = Lwt.wait () in
 
@@ -251,24 +253,23 @@ let packagekit_service config proxy version =
           else make_signal_request ITrans.s_Finished1 finish in
 
         let signals = finished_signal :: connect_error :: signals in
-        lwt () = signals |> Lwt_list.iter_p (fun request -> request switch trans_proxy) in
+        signals |> Lwt_list.iter_p (fun request -> request switch trans_proxy) >>= fun () ->
 
         (* Start operation *)
-        lwt () = cb switch trans_proxy in
+        cb switch trans_proxy >>= fun () ->
 
         (* Wait for Finished signal *)
         finished
-      finally
-        Lwt_switch.turn_off switch
+      )
   end
 
-let packagekit = ref (fun config ->
+let packagekit = ref (fun lang_spec ->
   let proxy = lazy (
-    match_lwt Dbus.system () with
-    | None ->
-        log_debug "Can't connect to system D-BUS; PackageKit support disabled";
-        Lwt.return None
-    | Some bus ->
+    Dbus.system () >>= function
+    | `Error reason ->
+        log_debug "Can't connect to system D-BUS; PackageKit support disabled (%s)" reason;
+        Lwt.return (`Unavailable (Printf.sprintf "PackageKit not available: %s" reason))
+    | `Ok bus ->
         let proxy = Dbus.OBus_proxy.make
           ~peer:(Dbus.OBus_peer.make ~connection:bus ~name:"org.freedesktop.PackageKit")
           ~path:["org"; "freedesktop"; "PackageKit"] in
@@ -277,7 +278,7 @@ let packagekit = ref (fun config ->
             Dbus.OBus_property.get (Dbus.OBus_property.make prop proxy)
           ) in
           Lwt_timeout.create 5 (fun () -> Lwt.cancel version) |> Lwt_timeout.start;
-          lwt version = version in
+          version >>= fun version ->
           let version = version |> List.map Int32.to_int in
           log_info "Found PackageKit D-BUS service, version %s" (String.concat "." (List.map string_of_int version));
 
@@ -287,15 +288,15 @@ let packagekit = ref (fun config ->
               0 :: version
             ) else version in
 
-          let p = packagekit_service config proxy version in
-          Lwt.return (Some p)
+          let p = packagekit_service lang_spec proxy version in
+          Lwt.return (`Ok p)
         with
         | Lwt.Canceled ->
             log_warning "Timed-out waiting for PackageKit to report its version number!";
-            Lwt.return None
+            Lwt.return (`Unavailable "Timed-out waiting for PackageKit to report its version number!")
         | Dbus.OBus_bus.Service_unknown msg | Dbus.OBus_error.Unknown_object msg ->
             log_info "PackageKit not available: %s" msg;
-            Lwt.return None
+            Lwt.return (`Unavailable (Printf.sprintf "PackageKit not available: %s" msg))
   ) in
 
   (* Send a single PackageKit transaction for these packages. *)
@@ -304,11 +305,13 @@ let packagekit = ref (fun config ->
 
     try_lwt
       (* Convert short names to PackageKit IDs *)
-      lwt resolutions = resolve proxy (List.map fst queries) in
+      resolve proxy (List.map fst queries) >>= fun resolutions ->
 
       let package_ids = StringMap.fold (fun _ infos acc ->
         infos |> List.fold_left (fun acc info ->
-          get_packagekit_id info.retrieval_method :: acc
+          match info with
+          | `Ok info -> get_packagekit_id info.retrieval_method :: acc
+          | `Error _ -> acc
         ) acc
       ) resolutions [] in
 
@@ -325,21 +328,32 @@ let packagekit = ref (fun config ->
           log_warning ~ex "packagekit: GetDetails failed";
           Lwt.return StringMap.empty in
 
-      (* Update [resolutions] with the size information *)
-      let add_size impl =
-        let rm = impl.retrieval_method in
-        let packagekit_id = get_packagekit_id rm in
-        match StringMap.find packagekit_id sizes with
-        | Some _ as size -> {impl with retrieval_method = {rm with Impl.distro_size = size}}
-        | None ->
-            log_info "No size returned for '%s'" packagekit_id;
-            impl in
-
       log_info "packagekit: fetch_batch done";
       (* Notify that each query is done *)
       queries |> List.iter (fun (name, resolver) ->
-        let impls = default [] @@ StringMap.find name resolutions in
-        Lwt.wakeup resolver (impls |> List.map add_size)
+        let impls = StringMap.find name resolutions |> default [] in
+        let problems = ref [] in
+
+        if impls = [] then
+          problems := Printf.sprintf "'%s' details not in PackageKit response" name :: !problems;
+
+        (* Update [resolutions] with the size information *)
+        let add_size = function
+          | `Error msg -> problems := Printf.sprintf "%s: %s" name msg :: !problems; None
+          | `Ok impl ->
+          let rm = impl.retrieval_method in
+          let packagekit_id = get_packagekit_id rm in
+          match StringMap.find packagekit_id sizes with
+          | Some _ as size -> Some {impl with retrieval_method = {rm with Impl.distro_size = size}}
+          | None ->
+              log_info "No size returned for '%s'" packagekit_id;
+              Some impl in
+
+        let results = U.filter_map add_size impls in
+        Lwt.wakeup resolver {
+          results;
+          problems = !problems;
+        }
       );
 
       Lwt.return ()
@@ -350,31 +364,32 @@ let packagekit = ref (fun config ->
       Lwt.return () in
 
   object (_ : packagekit)
-    val candidates : (string, package_info list Lwt.t) Hashtbl.t = Hashtbl.create 10
+    val candidates : (string, query_result Lwt.t) Hashtbl.t = Hashtbl.create 10
 
     (** Names of packages we're about to issue a query for and their resolvers/wakers. *)
     val mutable next_batch = []
 
-    method is_available =
-      lwt proxy = Lazy.force proxy in
-      Lwt.return (proxy <> None)
+    method status =
+      Lazy.force proxy >|= function
+      | `Ok _proxy -> `Ok
+      | `Unavailable _ as un -> un
 
     (** Add any cached candidates.
         The candidates are those discovered by a previous call to [check_for_candidates].
         @param package_name the distribution's name for the package *)
-    method get_impls package_name : package_info list =
-      let task = try Hashtbl.find candidates package_name with Not_found -> Lwt.return [] in
+    method get_impls package_name =
+      let task = try Hashtbl.find candidates package_name with Not_found -> Lwt.return { results = []; problems = [] } in
 
       match Lwt.state task with
-      | Lwt.Sleep -> []             (* Fetch still in progress *)
-      | Lwt.Fail _ -> []            (* Fetch failed *)
+      | Lwt.Sleep -> { results = []; problems = [] }                 (* Fetch still in progress *)
+      | Lwt.Fail ex -> { results = []; problems = [Printexc.to_string ex] }     (* Fetch failed *)
       | Lwt.Return packages -> packages
 
     (** Request information about this package from PackageKit. *)
     method check_for_candidates ~ui ~hint package_names =
-      match_lwt Lazy.force proxy with
-      | None -> Lwt.return ()
-      | Some proxy ->
+      Lazy.force proxy >>= function
+      | `Unavailable _ -> Lwt.return ()
+      | `Ok proxy ->
           let progress, set_progress = Lwt_react.S.create (Int64.zero, None, false) in
           try_lwt
             let waiting_for = ref [] in
@@ -432,20 +447,18 @@ let packagekit = ref (fun config ->
             Lwt.return ()
 
     method install_packages (ui:#ui) items : [ `ok | `cancel ] Lwt.t =
-      lwt response =
-        let packagekit_ids = items |> List.map (fun (_impl, rm) -> get_packagekit_id rm) in
-        ui#confirm (
-          "The following components need to be installed using native packages. \
-           These come from your distribution, and should therefore be trustworthy, but they also \
-           run with extra privileges. In particular, installing them may run extra services on your \
-           computer or affect other users. You may be asked to enter a password to confirm. The \
-           packages are:\n\n- " ^ (String.concat "\n- " packagekit_ids)
-        ) in
-      match response with
+      let packagekit_ids = items |> List.map (fun (_impl, rm) -> get_packagekit_id rm) in
+      ui#confirm (
+        "The following components need to be installed using native packages. \
+         These come from your distribution, and should therefore be trustworthy, but they also \
+         run with extra privileges. In particular, installing them may run extra services on your \
+         computer or affect other users. You may be asked to enter a password to confirm. The \
+         packages are:\n\n- " ^ (String.concat "\n- " packagekit_ids)
+      ) >>= function
       | `cancel -> Lwt.return `cancel
       | `ok ->
-          lwt pk = Lazy.force proxy in
-          let pk = pk |? lazy (failwith "BUG: PackageKit has disappeared!") in
-          install ui pk items
+          Lazy.force proxy >>= function
+          | `Ok pk -> install ui pk items
+          | `Unavailable _ -> failwith "BUG: PackageKit has disappeared!"
   end
 )
