@@ -9,11 +9,16 @@ type size = Int64.t
 
 type package_info = {
   version : Version.t;
-  machine : string option;
+  machine : Arch.machine option;
   installed : bool;
   retrieval_method : Impl.distro_retrieval_method;
 }
 
+type query_result = {
+  results : package_info list;
+  problems : string list;
+}
+
 class type ui =
   object
     method monitor : Downloader.download -> unit
@@ -22,22 +27,22 @@ class type ui =
   end
 
 type packagekit = <
-  (** Check whether PackageKit is available (only slow the first time) *)
-  is_available : bool Lwt.t;
+  (* Check whether PackageKit is available (only slow the first time) *)
+  status : [`Ok | `Unavailable of string] Lwt.t;
 
-  (** Return any cached candidates.
+  (* Return any cached candidates.
       The candidates are those discovered by a previous call to [check_for_candidates].
       @param package_name the distribution's name for the package *)
-  get_impls : string -> package_info list;
+  get_impls : string -> query_result;
 
-  (** Request information about these packages from PackageKit. *)
+  (* Request information about these packages from PackageKit. *)
   check_for_candidates : 'a. ui:(#ui as 'a) -> hint:string -> string list -> unit Lwt.t;
 
-  (** Install packages. Will confirm first with the user. *)
+  (* Install packages. Will confirm first with the user. *)
   install_packages : 'a. (#ui as 'a) -> (Impl.distro_implementation * Impl.distro_retrieval_method) list -> [ `ok | `cancel ] Lwt.t;
 >
 
 (** Create a packagekit object, which can be used to query the PackageKit D-BUS
  * service for information about (uninstalled) candidate packages.
  * (overridable for unit-tests) *)
-val packagekit : (General.config -> packagekit) ref
+val packagekit : (Support.Locale.lang_spec -> packagekit) ref
