@@ -22,7 +22,7 @@ val impl_provider : role -> Impl_provider.impl_provider
 
 (** Convert [Requirements.t] to requirements for the solver.
  * This looks at the host system to get some values (whether we have multi-arch support, default CPU and OS). *)
-val get_root_requirements : General.config -> Requirements.t -> (Impl_provider.scope_filter -> Impl_provider.impl_provider) -> Model.requirements
+val get_root_requirements : General.config -> Requirements.t -> (Scope_filter.t -> Impl_provider.impl_provider) -> Model.requirements
 
 (** Find a set of implementations which satisfy these requirements. Consider using [solve_for] instead.
     @param closest_match adds a lowest-ranked (but valid) implementation to every interface, so we can always
